@@ -96,120 +96,203 @@ void Board::GetPadWidths_MIL(std::list<int>& o, int& iDefaultWidth) const
 {
 	iDefaultWidth = GetPAD_MIL();
 	m_compMgr.GetPadWidths(o, iDefaultWidth);
+	if ( GetFatTracks() && std::find(o.begin(), o.end(), iDefaultWidth) == o.end() )
+		o.push_back( iDefaultWidth );
 }
 void Board::GetHoleWidths_MIL(std::list<int>& o, int& iDefaultWidth) const
 {
 	iDefaultWidth = GetHOLE_MIL();
 	m_compMgr.GetHoleWidths(o, iDefaultWidth);
 }
-double Board::GetMIN_TRACK_SEPARATION_MIL()	// Minimum track separation in mil
+void Board::GetSeparations(double& minTrackSeparation_mil, double& minGroundFill_mil)
 {
-	const double dMinSep	= GetMIN_SEPARATION();	// Min separation without ground fill
-	const double dGap		= GetGroundFill() ? GetGAP_MIL() : 100.0;
-	return std::min(dGap, dMinSep);
-}
-double Board::GetMIN_GROUNDFILL_MIL()	// Minimum ground-fill width in mil
-{
-	// To have a ground fill with no isolated islands this must be > 0 (and probably at least 8 mil)
-	if ( !GetGroundFill() ) return 100.0;
-	const double dMinSep	= GetMIN_SEPARATION();	// Min separation without ground fill
-	const double dDblGap	= GetGAP_MIL() * 2.0;
-	return std::max(0.0, dMinSep - dDblGap);
-}
-double Board::GetMIN_SEPARATION()	// Minimum separation (in mil) between a pad or track without ground fill
-{
-	// Following code applies to proper 2-layer routing.
-	// So wires have regular pad sizes, and are not converted to tracks on the top layer using a via.
+	if ( GetCompEdit() || GetVeroTracks() ) return;
 
-	const bool bDiagsOK  = GetDiagsMode() != DIAGSMODE::OFF;
-	const bool bMinDiags = GetDiagsMode() == DIAGSMODE::MIN;
+	// Calc minimum track separation in mil
+	const double dMinSep	= 100.0 * m_dMinSeparation;	// Min separation in MIL, without ground fill
+	const double dGap		= GetGroundFill() ? GetGAP_MIL() : 100.0;
+	minTrackSeparation_mil	= std::min(dGap, dMinSep);
+	// Calc minimum ground-fill width in mil
+	// To have a ground fill with no isolated islands, this must be > 0 (and probably at least 8 mil)
+	minGroundFill_mil		= GetGroundFill() ? std::max(0.0, dMinSep - dGap * 2.0) : 100.0;
+
+	// If the minimum track separation is determined by the gap,
+	// then all locations have min separation, so don't show warning points in the view
+	if ( minTrackSeparation_mil == dGap ) ClearWarnPoints();
+}
+void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
+{
+	ClearWarnPoints();	// Wipe list of warning locations on both layers
+	m_dMinSeparation = DBL_MAX;
+
+	if ( GetCompEdit() || GetVeroTracks() ) return;
+
+	const int nRings = 2;	// 2 ==> Max pad size supported by VeroRoute could be up to 200 mil in future
+	int Xmil, Ymil;	// For pad offsets
+
+	const bool bStandardBlobs = ( MAX_PAD_OFFSET_MIL <= 50 );	// true ==> legs for offset pads will be within a grid square
 
 	// Get bounds to minimise looping
 	int minRow, minCol, maxRow, maxCol;
 	GetBounds(minRow, minCol, maxRow, maxCol);
 
-	double dMin(100.0);
 	for (int k = 0, kMax = GetLyrs(); k < kMax; k++)	// Check all layers
-	for (int j = minRow; j <= maxRow; j++)
-	for (int i = minCol; i <= maxCol; i++)
 	{
-		const Element*	pA		= Get(k, j, i);
-		const int&		nodeIdA	= pA->GetNodeId();
-		if ( nodeIdA == BAD_NODEID ) continue;
-		const bool		bViaA	= pA->GetIsVia();
-		const bool		bPadA	= pA->GetHasPin();	assert( !(bViaA && bPadA) );
-		const bool		bWireA	= pA->GetHasWire();
+		PolygonHelper polygonHelper;
 
-		int iA(0);
-		if ( bWireA )
+		for (int j = minRow; j <= maxRow; j++)
+		for (int i = minCol; i <= maxCol; i++)
 		{
-			assert( bPadA );
-			iA = GetPAD_MIL();	// No custom pad size for wires
-		}
-		else if ( bPadA )
-		{
-			assert( pA->GetCompId() != BAD_COMPID );	// Non-wire part must use slot 0
-			const Component& comp	= m_compMgr.GetComponentById( pA->GetCompId() );
-			assert( comp.GetType() != COMP::INVALID );
-			iA = comp.GetCustomPads() ? comp.GetPadWidth() : GetPAD_MIL();
-		}
-		else if ( bViaA )	iA = GetVIAPAD_MIL();
-		else				iA = GetTRACK_MIL();
+			const Element*	pA		= Get(k, j, i);
+			const int&		nodeIdA	= pA->GetNodeId();
+			if ( nodeIdA == BAD_NODEID && !pA->GetHasPin() ) continue;	// Skip if no track and no pin
+
+			MyPointF pointA(i, j, 0.005 * GetTRACK_MIL());	// The blob centre for pA (note: track radius !!!)
+			MyPointF padA(pointA);							// The pad centre for pA (pad radius will be set below)
+			bool	 bPadA(true);							// Set false if there is no pad at pA
+
+			if ( pA->GetHasWire() )
+				padA.m_radius = 0.005 * GetPAD_MIL();	// No custom pad size for wires
+			else if ( pA->GetHasPin() )
+			{
+				const Component& comp	= m_compMgr.GetComponentById( pA->GetCompId() );	// Non-wire part must use slot 0
+				// Handle custom pad sizes
+				padA.m_radius = 0.005 * ( comp.GetCustomPads() ? comp.GetPadWidth() : GetPAD_MIL() );
+				// Handle pad offsets
+				comp.GetCompPinOffsets(pA->GetPinIndex(), Xmil, Ymil);
+				padA += QPointF(0.01 * Xmil, 0.01 * Ymil);
+			}
+			else if ( pA->GetIsVia() )
+				padA.m_radius = 0.005 * GetVIAPAD_MIL();
+			else
+				bPadA = false;
+
+			std::list<MyPolygonF> blobA;	// Blob A points (in units of grid squares)
+			CalcBlob(1, pointA, padA, GetPerimeterCode(pA), blobA, pA->GetHasPin());	// 1 ==> scale of 1 grid square
 
-		for (int iDiag = 0, iDiagMax = bDiagsOK ? 2 : 1; iDiag < iDiagMax; iDiag++)	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
-		{
-			const bool		bDiag	= ( iDiag == 1 );
-			const double	D		= bDiag ? ( 100.0 * sqrt(2.0) ) : 100.0;	// Distance to diagonal/HV nbr
-			const double	d		= D * 0.5;	// Only used when bDiag is true
 			// Only need to loop half the directions in the following loop (the i,j scan takes care of the other half)
-			for (int iNbr = iDiag; iNbr < 4; iNbr += 2)	// Even/Odd iNbr ==> Non-diagonal/Diagonal
+			for (int jj = std::max(minRow,j-nRings); jj <= j; jj++)
+			for (int ii = std::max(minCol,i-nRings), iiMax = std::min(maxCol,i+nRings); ii <= iiMax; ii++)
 			{
-				if ( !ReadCodeBit(iNbr, pA->GetRoutable()) ) continue;
-
-				const Element*	pB		= pA->GetNbr(iNbr);
+				if ( jj == j && ii == i ) continue;	// Skip pA
+				const Element*	pB		= Get(k, jj, ii);
 				const int&		nodeIdB	= pB->GetNodeId();
-				if ( nodeIdB == BAD_NODEID || nodeIdB == nodeIdA ) continue;
+				if ( nodeIdB == BAD_NODEID && !pB->GetHasPin() ) continue;	// Skip if no track and no pin
+				if ( nodeIdB == nodeIdA ) continue;
 
-				const bool bViaB	= pB->GetIsVia();
-				const bool bPadB	= pB->GetHasPin();	assert( !(bViaB && bPadB) );
-				const bool bWireB	= pB->GetHasWire();
+				MyPointF pointB(ii, jj, 0.005 * GetTRACK_MIL());	// The blob centre for pB (note: track radius !!!)
+				MyPointF padB(pointB);								// The pad centre for pB (pad radius will be set below)
+				bool	 bPadB(true);								// Set false if there is no pad at pB
 
-				int iB(0);
-				if ( bWireB )
+				if ( pB->GetHasWire() )
+					padB.m_radius = 0.005 * GetPAD_MIL();	// No custom pad size for wires
+				else if ( pB->GetHasPin() )
 				{
-					assert( bPadB );
-					iB = GetPAD_MIL();	// No custom pad size for wires
+					const Component& comp	= m_compMgr.GetComponentById( pB->GetCompId() );	// Non-wire part must use slot 0
+					// Handle custom pad sizes
+					padB.m_radius = 0.005 * ( comp.GetCustomPads() ? comp.GetPadWidth() : GetPAD_MIL() );
+					// Handle pad offsets
+					comp.GetCompPinOffsets(pB->GetPinIndex(), Xmil, Ymil);
+					padB += QPointF(0.01 * Xmil, 0.01 * Ymil);
 				}
-				else if ( bPadB )
-				{
-					assert( pB->GetCompId() != BAD_COMPID );	// Non-wire part must use slot 0
-					const Component& comp	= m_compMgr.GetComponentById( pB->GetCompId() );
-					assert( comp.GetType() != COMP::INVALID );
-					iB = comp.GetCustomPads() ? comp.GetPadWidth() : GetPAD_MIL();
-				}
-				else if ( bViaB )	iB = GetVIAPAD_MIL();
-				else				iB = GetTRACK_MIL();
+				else if ( pB->GetIsVia() )
+					padB.m_radius = 0.005 * GetVIAPAD_MIL();
+				else
+					bPadB = false;
 
-				dMin = std::min(dMin, D - 0.5 * ( iA + iB ));
-			}
-			if ( bDiag )
-			{
-				// Now consider connections between pA's non-diagonal nbrs (i.e. "adjacent diagonals")
-				bool bAdjDiag(false);
-				for (int iNbr = 0; iNbr < 8 && !bAdjDiag; iNbr += 2)	// Loop all non-diagonal nbrs
-				{
-					const Element*	pB		= pA->GetNbr(iNbr);
-					const int&		nodeIdB	= pB->GetNodeId();
-					if ( nodeIdB == BAD_NODEID || nodeIdB == nodeIdA ) continue;
-					const int iPerimeterCode = pB->GetPerimeterCode(bDiagsOK, bMinDiags);	// 0 to 255
-					// (iNbr+3)%8   NBR_L ==> NBR_RT,  NBR_T ==> NBR_RB, NBR_R ==> NBR_LB,  NBR_B ==> NBR_LT
-					bAdjDiag =  ReadCodeBit((iNbr+3) % 8, iPerimeterCode);
-				}
-				if ( bAdjDiag ) dMin = std::min(dMin, d - 0.5 * ( iA + GetTRACK_MIL() ));
+				std::list<MyPolygonF> blobB;	// Blob B points (in units of grid squares)
+				CalcBlob(1, pointB, padB, GetPerimeterCode(pB), blobB, pB->GetHasPin());	// 1 ==> scale of 1 grid square
+
+				const bool bCompareBlobs = !bStandardBlobs || ( abs(jj - j) < 2 && abs(ii - i) < 2 );	// Standard blobs ==> just consider neighbouring grid points
+
+				// Pad A to Pad B
+				if ( bPadA && bPadB ) polygonHelper.CalcSeparation(padA, padB);
+
+				// Pad A to Blob B
+				if ( bPadA ) for(auto& b : blobB) polygonHelper.CalcSeparation(padA, b);
+
+				// Pad B to Blob A
+				if ( bPadB ) for(auto& a : blobA) polygonHelper.CalcSeparation(padB, a);
+
+				// Blob A to Blob B
+				if ( bCompareBlobs ) for(auto & a : blobA) for(auto& b : blobB) polygonHelper.CalcSeparation(a, b);
 			}
 		}
+		if ( polygonHelper.m_Dmin > m_dMinSeparation ) continue;
+		if ( polygonHelper.m_Dmin < m_dMinSeparation )	// If min for layer is lowest across all layers ...
+			ClearWarnPoints();							// ... wipe all warning points
+
+		for (auto& p : polygonHelper.m_pWarn) m_warnPoints[k].push_back(p);
+		m_dMinSeparation = polygonHelper.m_Dmin;
+	}	// Next layer
+}
+
+void Board::CalcGroundFillBounds()
+{
+	const int&	W = GetGRIDPIXELS();	// Square width in pixels
+	const int	C = W >> 1;				// Half square width in pixels
+
+	int deltaL(0), deltaR(0), deltaT(0), deltaB(0);	// Maximum pad protrusions (in pixels) at edge of grid
+
+	// Loop points on edge of grid and look for large pads there
+	int minRow(0), minCol(0), maxRow(GetRows()-1), maxCol(GetCols()-1);
+	for (int j = minRow; j <= maxRow; j++)
+	{
+		const int iStep = ( j == minRow || j == maxRow ) ? 1 : std::max(1, maxCol - minCol);
+		for (int i = minCol; i <= maxCol; i += iStep)
+		{
+			const Element* p = Get(0, j, i);	// Sufficient to check layer 0 when looking for pins
+			if ( !p->GetHasPin() ) continue;
+			if ( p->GetHasWire() ) continue;	// Wires can't have custom sized pads or offset pads
+
+			const int	 compId		= p->GetCompId();	assert( compId != BAD_COMPID );
+			const size_t pinIndex	= p->GetPinIndex();	assert( pinIndex != BAD_PININDEX );
+
+			Component&	comp	= m_compMgr.GetComponentById( compId );
+			int Xmil(0), Ymil(0);	// Pad offsets
+			comp.GetCompPinOffsets(pinIndex, Xmil, Ymil);
+
+			if ( !comp.GetCustomPads() && Xmil == 0 && Ymil == 0) continue;	// Skip pad if not custom or offset
+
+			const int w			= comp.GetPadWidth();
+			const int padWidth	= GetHalfPixelsFromMIL( w ) + 1;	// +1 for back compatibility (i.e. max pad size was 98)
+			const int delta		= padWidth - C;		// The protrusion for a pad without offset
+
+			if ( i == minCol ) deltaL = std::max(deltaL, delta - ( Xmil * W ) / 100);
+			if ( i == maxCol ) deltaR = std::max(deltaR, delta + ( Xmil * W ) / 100);
+			if ( j == minRow ) deltaT = std::max(deltaT, delta - ( Ymil * W ) / 100);
+			if ( j == maxRow ) deltaB = std::max(deltaB, delta + ( Ymil * W ) / 100);
+		}
 	}
-	return std::max(0.0, dMin);
+	m_gndL = 0 - deltaL;
+	m_gndT = 0 - deltaT;
+	m_gndR = W * GetCols() + deltaR;
+	m_gndB = W * GetRows() + deltaB;
+
+	// Loop all shapes and get a safe estimate of their outer bounds
+	const bool bMonoPCB		= GetTrackMode() == TRACKMODE::MONO || GetTrackMode() == TRACKMODE::PCB;
+	const bool bFill		= !bMonoPCB && GetFillSaturation() > 0;	// No component fill in Mono/PCB mode
+	for (const auto& mapObj : m_compMgr.GetMapIdToComp())
+	{
+		const Component& comp = mapObj.second;
+
+		double L,R,T,B;
+		comp.GetSafeBounds(L, R, T, B, bFill);
+
+		// Footprint centre
+		const double X = C * ( comp.GetCol() + comp.GetLastCol() + 1 );
+		const double Y = C * ( comp.GetRow() + comp.GetLastRow() + 1 );
+
+		m_gndL = std::min(m_gndL, (int)(X + W * L));
+		m_gndT = std::min(m_gndT, (int)(Y + W * T));
+		m_gndR = std::max(m_gndR, (int)(X + W * R));
+		m_gndB = std::max(m_gndB, (int)(Y + W * B));
+	}
+}
+
+void Board::GetGroundFillBounds(int& L, int& R, int& T, int& B) const
+{
+	L = m_gndL; R = m_gndR; T = m_gndT; B = m_gndB;
 }
 
 // Methods to paint/unpaint nodeIds
@@ -232,24 +315,24 @@ void Board::SetNodeId(Element* p, const int& nodeId, const bool bAllLyrs)	// Hel
 	}
 }
 
-void Board::ClearFlagBits(Element* p, const char& i, const bool bAllLyrs)
+void Board::WipeFlagBits(Element* p, const char& i, const bool bAllLyrs)
 {
 	// For all layers case, always write base layer first
 	Element* q	= bAllLyrs ? p->GetNbr(NBR_X) : nullptr;
 	Element* p1	= std::min(p, q);
 	Element* p2	= std::max(p, q);
-	if ( p1 ) p1->ClearFlagBits(i);
-	if ( p2 ) p2->ClearFlagBits(i);
+	if ( p1 ) p1->WipeFlagBits(i);
+	if ( p2 ) p2->WipeFlagBits(i);
 }
 
-void Board::SetFlagBits(Element* p, const char& i, const bool bAllLyrs)
+void Board::MarkFlagBits(Element* p, const char& i, const bool bAllLyrs)
 {
 	// For all layers case, always write base layer first
 	Element* q	= bAllLyrs ? p->GetNbr(NBR_X) : nullptr;
 	Element* p1	= std::min(p, q);
 	Element* p2	= std::max(p, q);
-	if ( p1 ) p1->SetFlagBits(i);
-	if ( p2 ) p2->SetFlagBits(i);
+	if ( p1 ) p1->MarkFlagBits(i);
+	if ( p2 ) p2->MarkFlagBits(i);
 }
 
 bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, const int& nodeId, const bool& bPaintPins)
@@ -273,7 +356,6 @@ bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, cons
 
 		const size_t	pinIndex	= p->GetPinIndex();
 		const int&		compId		= p->GetCompId();
-		assert( compId != BAD_COMPID );
 		Component&		comp		= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
 		assert( comp.GetType() != COMP::WIRE );	// Sanity check
@@ -297,7 +379,6 @@ bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, cons
 		int		compId;
 		p->GetSlotInfo(p->GetUsedSlot(), pinIndex, compId);
 
-		assert( compId != BAD_COMPID );
 		const Component& comp	= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() == COMP::WIRE );	// Sanity check
 		const int		 origId	= comp.GetOrigId(lyr, pinIndex);
@@ -315,8 +396,8 @@ bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, cons
 		Element* pW = const_cast<Element*> (o.first);
 		const bool bAllLyrs = pW->GetHasPin();
 		SetNodeId(pW, nodeId, bAllLyrs);
-		ClearFlagBits(pW, AUTOSET|VEROSET, bAllLyrs);
-		SetFlagBits(pW, USERSET, bAllLyrs);
+		WipeFlagBits(pW, AUTOSET|VEROSET, bAllLyrs);
+		MarkFlagBits(pW, USERSET, bAllLyrs);
 	}
 
 	// Set the nodeId at the component pin
@@ -339,7 +420,6 @@ bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, cons
 
 					if ( pW == nullptr ) continue;
 
-					assert( compId != BAD_COMPID );
 					Component& comp = m_compMgr.GetComponentById( compId );
 					assert( comp.GetType() == COMP::WIRE );	// Sanity check
 					const size_t otherPinIndex = ( pinIndex == 0 ) ? 1 : 0;
@@ -359,7 +439,6 @@ bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, cons
 		{
 			const size_t	pinIndex	= p->GetPinIndex();
 			const int&		compId		= p->GetCompId();
-			assert( compId != BAD_COMPID );
 			Component&		comp		= m_compMgr.GetComponentById( compId );
 			assert( comp.GetType() != COMP::INVALID );
 			assert( bPaintPins && comp.GetType() != COMP::WIRE );	// Sanity check
@@ -391,7 +470,6 @@ void Board::FloodNodeId(const int& nodeId)
 			int compId;
 			p->GetSlotInfo(p->GetUsedSlot(), pinIndex, compId);
 
-			assert( compId != BAD_COMPID );
 			const Component& comp	= m_compMgr.GetComponentById( compId );
 			assert( comp.GetType() == COMP::WIRE );	// Sanity check
 			const int		 origId	= comp.GetOrigId(lyr, pinIndex);
@@ -469,8 +547,8 @@ void Board::AutoFillVero()
 				SetNodeId(pC, nodeIdTop, bAllLyrs);
 			else
 				SetNodeId(pC, nodeIdBot, bAllLyrs);
-			ClearFlagBits(pC, USERSET|AUTOSET, bAllLyrs);
-			SetFlagBits(pC, VEROSET, bAllLyrs);
+			WipeFlagBits(pC, USERSET|AUTOSET, bAllLyrs);
+			MarkFlagBits(pC, VEROSET, bAllLyrs);
 
 			nodeIdTop = pC->GetNodeId(); lenTop = 1;	// Start top count
 		}
@@ -499,8 +577,8 @@ void Board::CalcSolder()	// Work out locations of solder blobs to join veroboard
 void Board::SetSolder(const int& nodeId, const int& col, const bool& bVertical)
 {
 	// Sets elements of bSolderR to indicate solder blob between col and col+1
-	if (  bVertical && col == GetCols()-1 ) return;
-	if ( !bVertical && col == GetRows()-1 ) return;
+	const int colMax = ( bVertical ) ? GetCols() : GetRows();
+	if ( col == colMax-1 ) return;
 
 	assert( nodeId != BAD_NODEID );
 	int bestRow(-1), bestRowPins(-INT_MAX), bestRowPads(INT_MAX);
@@ -522,7 +600,6 @@ void Board::SetSolder(const int& nodeId, const int& col, const bool& bVertical)
 					rowPins++;
 				else
 				{
-					assert( pC->GetCompId() != BAD_COMPID );
 					const Component& comp = m_compMgr.GetComponentById( pC->GetCompId() );
 					assert( comp.GetType() != COMP::INVALID );
 					if ( comp.GetType() == COMP::PAD )
@@ -537,7 +614,6 @@ void Board::SetSolder(const int& nodeId, const int& col, const bool& bVertical)
 					rowPins++;
 				else
 				{
-					assert( pR->GetCompId() != BAD_COMPID );
 					const Component& comp = m_compMgr.GetComponentById( pR->GetCompId() );
 					assert( comp.GetType() != COMP::INVALID );
 					if ( comp.GetType() == COMP::PAD )
@@ -560,10 +636,31 @@ void Board::SetSolder(const int& nodeId, const int& col, const bool& bVertical)
 		{
 			if ( bestRow != -1 )
 			{
-				if ( bVertical )
-					Get(lyr, bestRow, col)->SetSolderR(true);
-				else
-					Get(lyr, col, bestRow)->SetSolderR(true);
+				// Finally check that there isn't a wire joining the strips
+				bool bHaveWire(false);	// Set true if a wire joins the strips
+				for (int r = bestRow; r >= 0 && !bHaveWire; r--)		// Search one way ...
+				{
+					const Element*	pC	= bVertical ? Get(lyr, r, col)		: Get(lyr, col,   r);
+					if ( pC->GetNodeId() != nodeId ) break;
+					const Element*	pR	= bVertical ? Get(lyr, r, col+1)	: Get(lyr, col+1, r);
+					if ( pR->GetNodeId() != nodeId ) break;
+					bHaveWire = ( pC->GetW(0) == pR || pC->GetW(1) == pR );
+				}
+				for (int r = bestRow; r < rowMax && !bHaveWire; r++)	// ... and the other
+				{
+					const Element*	pC	= bVertical ? Get(lyr, r, col)		: Get(lyr, col,   r);
+					if ( pC->GetNodeId() != nodeId ) break;
+					const Element*	pR	= bVertical ? Get(lyr, r, col+1)	: Get(lyr, col+1, r);
+					if ( pR->GetNodeId() != nodeId ) break;
+					bHaveWire = ( pC->GetW(0) == pR || pC->GetW(1) == pR );
+				}
+				if ( !bHaveWire )
+				{
+					if ( bVertical )
+						Get(lyr, bestRow, col)->SetSolderR(true);
+					else
+						Get(lyr, col, bestRow)->SetSolderR(true);
+				}
 			}
 			bestRow		= -1;		// Reset
 			bestRowPins	= -INT_MAX;	// Reset
@@ -581,7 +678,7 @@ bool Board::GetDisableCompText()
 	if ( ( GetGroupMgr().GetNumUserComps() != 1 ) || ( GetCompMode() == COMPSMODE::OFF || GetCompMode() == COMPSMODE::OUTLINE ) ) return true;
 	const Component& comp	= GetUserComponent();
 	const COMP&		 eType	= comp.GetType();
-	return ( eType == COMP::WIRE || eType == COMP::MARK );	// No labels for wires and markers
+	return ( eType == COMP::WIRE || eType == COMP::MARK || eType == COMP::VERO_NUMBER || eType == COMP::VERO_LETTER );	// No labels for wires/markers/vero-labels
 }
 
 bool Board::GetDisableMove()
