@@ -25,6 +25,9 @@
 #include "GroupManager.h"
 #include "ColorManager.h"
 #include "TextManager.h"
+#include "PolygonHelper.h"
+
+#define SMART_PAN_CAN_AUTOCROP true
 
 class MyScrollArea;
 
@@ -175,6 +178,64 @@ public:
 		}
 	}
 
+	bool ToggleLyrPref(int iLyr, int iRow, int iCol, bool bReset)
+	{
+		Element* p = Get(iLyr, iRow, iCol);
+		if ( !p->GetHasPin() || p->GetHasWire() ) return false;	// Want pins only.  Hole sharing wires could contradict each other, so ignore them
+
+		const int&		compId		= p->GetCompId();	assert(compId != BAD_COMPID);
+		const size_t	pinIndex	= p->GetPinIndex();	assert(pinIndex != BAD_PININDEX);
+		Component&		comp		= m_compMgr.GetComponentById(compId);
+		const uchar		oldPref		= comp.GetLayerPref(pinIndex);
+		uchar			newPref(LAYER_X);
+		if ( !bReset )
+		{
+			switch( oldPref )
+			{
+				case LAYER_X:	newPref = ( iLyr == 0 ) ? LAYER_B : LAYER_T;	break;
+				case LAYER_B:	newPref = LAYER_T;	break;
+				case LAYER_T:	newPref = LAYER_B;	break;
+			}
+		}
+		const bool bChanged = ( newPref != oldPref );
+		if ( bChanged ) comp.SetLayerPref(pinIndex, newPref);
+		return bChanged;
+	}
+
+	int GetPerimeterCode(const Element* p)	// Helper for the GUI "blobs"
+	{
+		const bool	bDiagsOK	= GetDiagsMode() != DIAGSMODE::OFF;
+		const bool	bMinDiags	= GetDiagsMode() == DIAGSMODE::MIN;
+
+		int iCode = p->GetPerimeterCode(bDiagsOK, bMinDiags);	// 0 to 255
+
+		// For a 2-layer board, modify perimeter code to handle pin layer preferences
+		if ( GetLyrs() == 1 || !p->GetHasPin() || p->GetHasWire() ) return iCode;
+
+		const bool		bBottomLayer	= p->IsLayer0();	// true ==> p is on bottom layer
+		const int&		compId			= p->GetCompId();		assert(compId != BAD_COMPID);
+		const size_t	pinIndex		= p->GetPinIndex();		assert(pinIndex != BAD_PININDEX);
+		const int		iLayerPrefP		= m_compMgr.GetComponentById(compId).GetLayerPref(pinIndex);
+
+		for (int iNbr = 0; iNbr < 8; iNbr ++)
+		{
+			if ( !ReadCodeBit(iNbr, iCode) ) continue;
+
+			const Element* q = p->GetNbr(iNbr);
+			if ( !q->GetHasPin() || q->GetHasWire() ) continue;
+
+			const int&		compId		= q->GetCompId();		assert(compId != BAD_COMPID);
+			const size_t	pinIndex	= q->GetPinIndex();		assert(pinIndex != BAD_PININDEX);
+			const int		iLayerPrefQ	= m_compMgr.GetComponentById(compId).GetLayerPref(pinIndex);
+
+			const bool bOK = ( iLayerPrefP == LAYER_X && iLayerPrefQ == LAYER_X ) ||
+							 ( bBottomLayer ? ( iLayerPrefP == LAYER_B || iLayerPrefQ == LAYER_B )
+											: ( iLayerPrefP == LAYER_T || iLayerPrefQ == LAYER_T ) );
+			if ( !bOK ) ClearCodeBit(iNbr, iCode);
+		}
+		return iCode;
+	}
+
 	void GlueWires()	// Set pointers between wired grid elements
 	{
 		for (const auto& mapObj : m_compMgr.GetMapIdToComp())	// Iterate components
@@ -253,21 +314,27 @@ public:
 			return GrowThenPan(0, incRows, incCols, 0, 0);
 		}
 
-		if ( minRow + iDown  < 0 )							// Too far up ...
+		if ( minRow + iDown < 0 )								// Too far up ...
 		{
-			iDown = -minRow;								// ... so pan to very top
-			incRows = std::min(0, maxRow + 1 - GetRows());	// ... and shrink grid from bottom
+			if ( SMART_PAN_CAN_AUTOCROP )
+				incRows = std::min(0, maxRow + 1 - GetRows());	// ... so shrink grid from bottom
+			//else
+			//	incRows	= -(minRow + iDown);					// ... so grow grid from bottom
+			iDown	= -minRow;									// ... and pan to very top
 		}
-		else if ( maxRow + iDown  >= GetRows() )			// Too far down ...
-			incRows	= 1 + maxRow + iDown - GetRows();		// ...so grow grid from bottom
+		else if ( maxRow + iDown >= GetRows() )					// Too far down ...
+			incRows	= 1 + maxRow + iDown - GetRows();			// ...so grow grid from bottom
 
-		if ( minCol + iRight < 0 )							// Too far left ...
+		if ( minCol + iRight < 0 )								// Too far left ...
 		{
-			iRight	= -minCol;								// ... so pan to very left
-			incCols	= std::min(0, maxCol + 1 - GetCols());	// ... and shrink grid from right
+			if ( SMART_PAN_CAN_AUTOCROP )
+				incCols	= std::min(0, maxCol + 1 - GetCols());	// ... so shrink grid from right
+			//else
+			//	incCols	= -(minCol + iRight);					// ... so grow grid from right
+			iRight	= -minCol;									// ... and pan to very left
 		}
-		else if ( maxCol + iRight >= GetCols() )			// Too far right ...
-			incCols	= 1 + maxCol + iRight - GetCols();		// ... so grow grid from right
+		else if ( maxCol + iRight >= GetCols() )				// Too far right ...
+			incCols	= 1 + maxCol + iRight - GetCols();			// ... so grow grid from right
 
 		GrowThenPan(0, incRows, incCols, iDown, iRight);
 	}
@@ -343,6 +410,10 @@ public:
 				PutDown(comp);
 			}
 		}
+
+		// Move any warning points
+		for (auto& o : m_warnPoints[0])	o += QPointF(iRight, iDown);
+		for (auto& o : m_warnPoints[1])	o += QPointF(iRight, iDown);
 	}
 
 	bool GetBounds(int& minRow, int& minCol, int& maxRow, int& maxCol) const
@@ -356,7 +427,7 @@ public:
 		for (int j = 0; j < numRows; j++)
 		for (int i = 0; i < numCols; i++)
 		{
-			if ( Get(k,j,i)->GetNodeId() == BAD_NODEID ) continue;
+			if ( Get(k,j,i)->GetNodeId() == BAD_NODEID && !Get(k,j,i)->GetHasPin() ) continue;
 			minRow = std::min(minRow, j);	maxRow = std::max(maxRow, j);
 			minCol = std::min(minCol, i);	maxCol = std::max(maxCol, i);
 			bOK = true;
@@ -387,7 +458,6 @@ public:
 	Component& GetUserComponent()	// The currently selected component
 	{
 		assert( m_groupMgr.GetNumUserComps() == 1 );	// Should only have one component selected
-		assert( m_groupMgr.GetUserCompId() != BAD_COMPID );
 		Component& comp = m_compMgr.GetComponentById( m_groupMgr.GetUserCompId() );
 		assert( comp.GetType() != COMP::INVALID );
 		return comp;
@@ -400,14 +470,15 @@ public:
 	// Methods to handle variable pad/hole and PCB tolerances
 	void	GetPadWidths_MIL(std::list<int>& o, int& iDefaultWidth) const;
 	void	GetHoleWidths_MIL(std::list<int>& o, int& iDefaultWidth) const;
-	double	GetMIN_TRACK_SEPARATION_MIL();	// Minimum guaranteed track separation in mil
-	double	GetMIN_GROUNDFILL_MIL();		// Minimum guaranteed ground-fill width in mil
-	double	GetMIN_SEPARATION();			// Minimum possible separation (in mil) between a pad or track without ground fill
+	void	GetSeparations(double& minTrackSeparation_mil, double& minGroundFill_mil);
+	void	CalcMIN_SEPARATION();	// Sets m_dMinSeparation and m_warnPoints[]
+	void	CalcGroundFillBounds();
+	void	GetGroundFillBounds(int& L, int& R, int& T, int& B) const;
 
 	// Methods to paint/unpaint nodeIds
 	void SetNodeId(Element* p, const int& nodeId, const bool bAllLyrs);	// Helper to make sure we do UpdateCounts() before painting an element
-	void ClearFlagBits(Element* p, const char& i, const bool bAllLyrs);
-	void SetFlagBits(Element* p, const char& i, const bool bAllLyrs);
+	void WipeFlagBits(Element* p, const char& i, const bool bAllLyrs);
+	void MarkFlagBits(Element* p, const char& i, const bool bAllLyrs);
 	bool SetNodeIdByUser(const int& lyr, const int& row, const int& col, const int& nodeId, const bool& bPaintPins);
 	void FloodNodeId(const int& nodeId);
 	void AutoFillVero();
@@ -495,6 +566,11 @@ public:
 	bool SetCurrentPinId(const int& i)		{ return m_compDefiner.SetCurrentPinId(i); }
 	bool SetCurrentShapeId(const int& i)	{ return m_compDefiner.SetCurrentShapeId(i); }
 
+	// Helpers for locations of close tracks
+	std::list<QPointF>& GetWarnPoints(int iLayer)	{ return m_warnPoints[iLayer]; }
+	void ClearWarnPoints()							{ m_warnPoints[0].clear(); m_warnPoints[1].clear(); }
+	bool GetHaveWarnPoints() const					{ return !m_warnPoints[0].empty() || !m_warnPoints[1].empty(); }
+
 	// Import Protel V1 / Tango netlist (exported from TinyCAD / gEDA)
 	bool ImportTango(const TemplateManager& templateMgr, const std::string& filename, std::string& errorStr);
 	// Import OrcadPCB2 netlist (exported from KiCAD)
@@ -675,7 +751,17 @@ private:
 	TextManager				m_textMgr;		// Handles the set of user-defined text labels
 	ColorManager			m_colorMgr;		// Handles color assignment to nodeIds
 
-	// Routing algorithm // Don't persist or copy
+	// Pixmap info for ground fill extent	// Don't persist or copy
+	int						m_gndL = 0;		// Set by CalcGroundFillBounds()
+	int						m_gndR = 0;		// Set by CalcGroundFillBounds()
+	int						m_gndT = 0;		// Set by CalcGroundFillBounds()
+	int						m_gndB = 0;		// Set by CalcGroundFillBounds()
+
+	// Track warnings		// Don't persist or copy
+	std::list<QPointF>		m_warnPoints[2];// For showing locations with min track separation on the 2 layers
+	double					m_dMinSeparation = DBL_MAX;	// Units of grid squares
+
+	// Routing algorithm	// Don't persist or copy
 	std::vector<Element*>	m_targetPins;	// Set of pins to route.
 	std::vector<Element*>	m_tmpVec;		// The set of visited points.
 	size_t					m_tmpVecSize;	// The number of visited points.
