@@ -74,7 +74,6 @@ int Board::AddComponent(int iRow, int iCol, const Component& tmp, bool bDoPlace)
 	const int compId = m_compMgr.CreateComp(tmp, GetUsePCBshapes() );	// CompMgr makes a copy of tmp and returns its compId
 	if ( compId == INT_MAX ) return BAD_COMPID;		// Reached component limit !!!
 
-	assert( compId != BAD_COMPID );
 	Component& comp = m_compMgr.GetComponentById( compId );
 	assert( comp.GetType() != COMP::INVALID );
 
@@ -271,7 +270,7 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 
 					// Check bottom grid
 					const int& nodeId		= pGrid->GetNodeId();		// Read nodeID on board
-					const int& iCompNodeId	= comp.GetNodeId(pinIndex);	// Read component nodeiD
+					const int& iCompNodeId	= comp.GetNodeId(pinIndex);	// Read component nodeID
 
 					if ( bWire )	// Wires have no NodeId. Need matching IDs on both ends
 						assert( iCompNodeId == BAD_NODEID );	// Shouldn't have an ID yet
@@ -402,9 +401,9 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 
 				const bool bAllLyrs(false);
 				if ( bExistingNodeId )						// If the board already had the NodeId ...
-					SetFlagBits(pGrid, RECTSET, bAllLyrs);	// ... set RECTSET on the board, so TakeOff() doesn't wipe the point
+					MarkFlagBits(pGrid, RECTSET, bAllLyrs);	// ... set RECTSET on the board, so TakeOff() doesn't wipe the point
 				else
-					ClearFlagBits(pGrid, RECTSET, bAllLyrs);
+					WipeFlagBits(pGrid, RECTSET, bAllLyrs);
 
 				// Fix up crossing diagonals
 				if ( bDiagsOK && j > 0 && i > 0 && pComp->GetUsed(iTraxNbrLT) != pGrid->GetUsed(NBR_LT) )
@@ -467,8 +466,8 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 				{
 					const bool bAllLyrs = pGrid->GetHasPin();	assert( bAllLyrs );
 					SetNodeId(pGrid, iCompNodeId, bAllLyrs);
-					ClearFlagBits(pGrid, AUTOSET|VEROSET, bAllLyrs);
-					SetFlagBits(pGrid, USERSET, bAllLyrs);
+					WipeFlagBits(pGrid, AUTOSET|VEROSET, bAllLyrs);
+					MarkFlagBits(pGrid, USERSET, bAllLyrs);
 				}
 			}
 		}
@@ -483,28 +482,32 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 		pA->SetW(iSlotA, pB);	// Link wire ends
 		pB->SetW(iSlotB, pA);	// Link wire ends
 
-		// Write nodeIds & flags
-		const bool bAllLyrs(true);
-		SetNodeId(pA, wireNodeId, bAllLyrs); SetFlagBits(pA, pB->GetFlag() & (USERSET|AUTOSET|VEROSET), bAllLyrs);
-		SetNodeId(pB, wireNodeId, bAllLyrs); SetFlagBits(pB, pA->GetFlag() & (USERSET|AUTOSET|VEROSET), bAllLyrs);
-
-		const char& iFlag = pA->GetFlag();
-
-		comp.SetNodeId(0, wireNodeId);
-		comp.SetNodeId(1, wireNodeId);
+		// Take logical OR of flags on wire ends, ignoring the RECTSET bit
+		const char iWireFlag = ( pA->GetFlag() | pB->GetFlag() ) & (USERSET|AUTOSET|VEROSET);
 
 		WIRELIST wireList;	// Helper for chains of wires
 
 		// Handle all connected wires.  (We just need the wirelist on one end, so use pA)
 		pA->GetWireList(wireList);	// Get list containing pA and its wired points
-		if ( wireList.size() > 2 )
+
+		for (auto& o : wireList)
 		{
-			for (auto& o : wireList)
+			Element* pW = const_cast<Element*> (o.first);
+			// Set the nodeId's on the wire components ...
+			for (int iSlot = 0; iSlot < 2; iSlot++)
 			{
-				Element* p = const_cast<Element*> (o.first);
-				if ( p == pA || p == pB ) continue;	// Can skip pA and pB (we've done these already)
-				SetNodeId(p, wireNodeId, bAllLyrs); SetFlagBits(p, iFlag & (USERSET|AUTOSET|VEROSET), bAllLyrs);
+				size_t	iPinIndex;
+				int		tmpCompId;
+				pW->GetSlotInfo(iSlot, iPinIndex, tmpCompId);
+				if ( iPinIndex == BAD_PININDEX ) continue;
+				Component& comp = m_compMgr.GetComponentById( tmpCompId );
+				assert( comp.GetType() == COMP::WIRE );
+				comp.SetNodeId(iPinIndex, wireNodeId);
 			}
+			// ... and on the corresponding board points
+			const bool bAllLyrs(true);
+			SetNodeId(pW, wireNodeId, bAllLyrs);
+			MarkFlagBits(pW, iWireFlag, bAllLyrs);
 		}
 	}
 	comp.SetIsPlaced(true);	
@@ -564,7 +567,7 @@ bool Board::TakeOff(Component& comp)
 				if ( pComp->GetNodeId() == BAD_NODEID ) continue;	// Skip blank areas of the trax comp
 				if ( !pGrid->ReadFlagBits(RECTSET) && ( !pGrid->GetHasPin() || pGrid->GetHasWire() ) )
 					SetNodeIdByUser(lyr, jRow, iCol, BAD_NODEID, false);	// false ==> don't paint pins
-				ClearFlagBits(pGrid, RECTSET, bAllLyrs);
+				WipeFlagBits(pGrid, RECTSET, bAllLyrs);
 			}
 			else
 			{
@@ -603,8 +606,8 @@ bool Board::TakeOff(Component& comp)
 						if ( p == nullptr ) continue;
 						const bool bAllLyrs = false;
 						SetNodeId(p, origId[lyr], bAllLyrs);	// Restore grid element to original nodeId
-						ClearFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
-						SetFlagBits(p, USERSET, bAllLyrs);
+						WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
+						MarkFlagBits(p, USERSET, bAllLyrs);
 					}
 				}
 			}
@@ -633,7 +636,6 @@ bool Board::TakeOff(Component& comp)
 				{
 					pW->GetSlotInfo(iSlot, iPinIndex, tmpCompId);
 					if ( iPinIndex == BAD_PININDEX ) continue;
-					assert( tmpCompId != BAD_COMPID );
 					const Component& comp = m_compMgr.GetComponentById( tmpCompId );
 					assert( comp.GetType() == COMP::WIRE );
 					origId0 = comp.GetOrigId(0, iPinIndex);
@@ -654,15 +656,14 @@ bool Board::TakeOff(Component& comp)
 					{
 						pW->GetSlotInfo(iSlot, iPinIndex, tmpCompId);
 						if ( iPinIndex == BAD_PININDEX ) continue;
-						assert( tmpCompId != BAD_COMPID );
 						Component& comp = m_compMgr.GetComponentById( tmpCompId );
-						assert( comp.GetType() != COMP::INVALID );
+						assert( comp.GetType() == COMP::WIRE );
 						comp.SetNodeId(iPinIndex, BAD_NODEID);
 					}
 					// ... and on the corresponding board points
 					SetNodeId(pW, BAD_NODEID, bAllLyrs);
-					ClearFlagBits(pW, AUTOSET|VEROSET, bAllLyrs);
-					SetFlagBits(pW, USERSET, bAllLyrs);
+					WipeFlagBits(pW, AUTOSET|VEROSET, bAllLyrs);
+					MarkFlagBits(pW, USERSET, bAllLyrs);
 				}
 			}
 		}
@@ -675,8 +676,8 @@ bool Board::TakeOff(Component& comp)
 				Element* p = ( lyr == 0 ) ? pA : pA->GetNbr(NBR_X);
 				if ( p == nullptr ) continue;
 				SetNodeId(p, iOrigIdA[lyr], bAllLyrs);	// Restore grid element to original nodeId
-				ClearFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
-				SetFlagBits(p, USERSET, bAllLyrs);
+				WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
+				MarkFlagBits(p, USERSET, bAllLyrs);
 			}
 		}
 		if ( !pB->GetHasWire() )	// If we've taken off the last wire at the location
@@ -687,8 +688,8 @@ bool Board::TakeOff(Component& comp)
 				Element* p = ( lyr == 0 ) ? pB : pB->GetNbr(NBR_X);
 				if ( p == nullptr ) continue;
 				SetNodeId(p, iOrigIdB[lyr], bAllLyrs);	// Restore grid element to original nodeId
-				ClearFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
-				SetFlagBits(p, USERSET, bAllLyrs);
+				WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
+				MarkFlagBits(p, USERSET, bAllLyrs);
 			}
 		}
 	}
@@ -754,15 +755,16 @@ void Board::SelectAllComps(bool bRestrictToRects)
 	SetCurrentTextId(BAD_TEXTID);
 }
 
-bool Board::ConfirmDestroyUserComps()	// returns false if user-group is empty or has only wires and markers
+bool Board::ConfirmDestroyUserComps()	// returns false if user-group is empty or has only wires/markers/vero-labels
 {
 	std::list<int> userCompIds;
 	m_groupMgr.GetUserCompIds(userCompIds);
 	for (const auto& compId : userCompIds)
 	{
-		assert( compId != BAD_COMPID );
 		switch( m_compMgr.GetComponentById( compId ).GetType() )
 		{
+			case COMP::VERO_NUMBER:
+			case COMP::VERO_LETTER:
 			case COMP::WIRE:
 			case COMP::MARK:	continue;
 			case COMP::INVALID:	assert(0);
@@ -779,7 +781,6 @@ void Board::DestroyUserComps()	// Destroy components in the user-group
 	m_groupMgr.GetUserCompIds(userCompIds);
 	for (const auto& compId : userCompIds)
 	{
-		assert( compId != BAD_COMPID );
 		Component& comp = m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
 		DestroyComponent( comp );
@@ -930,7 +931,6 @@ void Board::CopyComps(const std::list<int>& compIds)	// Make a blank copy of the
 	if ( bMakeNewGroup && newGroupId == INT_MAX ) return;	// Fail if we've reached the max allowed groupId
 	for (auto& compId : compIds)
 	{
-		assert( compId != BAD_COMPID );
 		const Component&	comp		= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
 		const int			newCompId	= CreateComponent(-1, -1, comp.GetType(), &comp);	// Create blank copy of the component and get its compId
@@ -993,7 +993,6 @@ bool Board::MoveComps(const std::list<int>& compIds, const int& deltaRow, const
 	// Take off all comps, and finally the trax comp. Move them but keep them floating.
 	for (auto& compId : compIds)
 	{
-		assert( compId != BAD_COMPID );
 		Component& comp = m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
 		int newRow = comp.GetRow() + deltaRow;
@@ -1022,7 +1021,6 @@ bool Board::MoveComps(const std::list<int>& compIds, const int& deltaRow, const
 	{
 		for (auto& compId : compIds)
 		{
-			assert( compId != BAD_COMPID );
 			Component& comp = m_compMgr.GetComponentById( compId );
 			assert( comp.GetType() != COMP::INVALID );
 			PutDown(comp);
@@ -1073,7 +1071,6 @@ void Board::RotateComps(const std::list<int>& compIds, const bool& bCW)	// Rotat
 	// Take off all comps, and finally the trax comp. Move and rotate them but keep them floating.
 	for (auto& compId : compIds)
 	{
-		assert( compId != BAD_COMPID );
 		Component& comp	= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
 		TakeOff(comp);
@@ -1123,7 +1120,6 @@ void Board::RotateComps(const std::list<int>& compIds, const bool& bCW)	// Rotat
 	{
 		for (auto& compId : compIds)
 		{
-			assert( compId != BAD_COMPID );
 			Component& comp = m_compMgr.GetComponentById( compId );
 			assert( comp.GetType() != COMP::INVALID );
 			PutDown(comp);
@@ -1143,8 +1139,8 @@ void Board::FixCorruption()
 
 	for (auto& compId : badCompIds)
 	{
-		assert( compId != BAD_COMPID );
 		Component& comp = m_compMgr.GetComponentById( compId );
+		assert( comp.GetType() == COMP::INVALID );
 		DestroyComponent(comp);
 	}
 
@@ -1181,7 +1177,6 @@ Rect Board::GetFootprintBounds(const std::list<int>& compIds)
 	Rect bounding;
 	for (auto& compId : compIds)
 	{
-		assert( compId != BAD_COMPID );
 		const Component& comp	= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
 		bounding |= comp.GetFootprintRect();
