@@ -308,7 +308,7 @@ public:
 	}
 	bool GetWireCanBeTrack(const Component* pWire) const	// true ==> wire can be turned into a top-surface track
 	{
-		assert( pWire->getType == COMP::WIRE && pWire->GetIsPlaced() );	// Should have already checked for this
+		assert( pWire->GetType() == COMP::WIRE && pWire->GetIsPlaced() );	// Should have already checked for this
 		// Wire must be non-stacked, and either horizontal or not crossing another
 		auto iter = m_mapWireToInfo.find( pWire );
 		const bool bH = pWire->GetDirection() == 'W' || pWire->GetDirection() == 'E';
@@ -328,8 +328,9 @@ public:
 		if ( str.empty() ) return;
 		for (auto& mapObj : m_mapIdToComp)
 		{
-			Component& comp = mapObj.second;
-			if ( comp.GetType() == COMP::WIRE || comp.GetType() == COMP::MARK ) continue;
+			Component&	comp	= mapObj.second;
+			const COMP&	eType	= comp.GetType();
+			if ( eType == COMP::WIRE || eType == COMP::MARK || eType == COMP::VERO_NUMBER || eType == COMP::VERO_LETTER ) continue;
 			const std::string& compStr = bUseName ? comp.GetNameStr() : comp.GetValueStr();
 			const bool bFound = bExact ? ( compStr == str )
 									   : ( compStr.find(str) != std::string::npos );
