@@ -23,6 +23,17 @@
 #include "RectManager.h"
 #include "CompDefiner.h"
 
+
+// For 2-layer boards, the following says which is the prefered layer for a pin to make connections.
+// This only affects rendered connections between adjacent pins..
+// It does not affect connectivity or routing.
+// The joint preference of adjacent pins determines which layers will show the connection.
+static const uchar	LAYER_X = 0;	// No preference
+static const uchar	LAYER_B	= 1;	// Prefer bottom layer
+static const uchar	LAYER_T	= 2;	// Prefer top layer
+
+static const int MAX_PAD_OFFSET_MIL = 50;
+
 class CompManager;
 
 // Class to describe a component.
@@ -48,6 +59,9 @@ public:
 		m_nodeIdPins.clear();
 		m_origIdPins1.clear();
 		m_origIdPins2.clear();
+		m_layerPrefs.clear();
+		m_pinOffsetRow.clear();
+		m_pinOffsetCol.clear();
 		m_pinLabels.clear();
 		m_pinAligns.clear();
 		m_shapes.clear();
@@ -131,6 +145,9 @@ public:
 		std::copy(o.m_nodeIdPins.begin(),	o.m_nodeIdPins.end(),	m_nodeIdPins.begin());
 		std::copy(o.m_origIdPins1.begin(),	o.m_origIdPins1.end(),	m_origIdPins1.begin());
 		std::copy(o.m_origIdPins2.begin(),	o.m_origIdPins2.end(),	m_origIdPins2.begin());
+		std::copy(o.m_layerPrefs.begin(),	o.m_layerPrefs.end(),	m_layerPrefs.begin());
+		std::copy(o.m_pinOffsetRow.begin(),	o.m_pinOffsetRow.end(),	m_pinOffsetRow.begin());
+		std::copy(o.m_pinOffsetCol.begin(),	o.m_pinOffsetCol.end(),	m_pinOffsetCol.begin());
 		std::copy(o.m_pinLabels.begin(),	o.m_pinLabels.end(),	m_pinLabels.begin());
 		std::copy(o.m_pinAligns.begin(),	o.m_pinAligns.end(),	m_pinAligns.begin());
 		CopyShapes( o );
@@ -141,6 +158,7 @@ public:
 		for (auto& o : m_nodeIdPins)  o = BAD_NODEID;
 		for (auto& o : m_origIdPins1) o = BAD_NODEID;
 		for (auto& o : m_origIdPins2) o = BAD_NODEID;
+		for (auto& o : m_layerPrefs)  o = LAYER_X;
 	}
 	bool IsEqual(const Component& o) const	// Compare persisted info
 	{
@@ -165,11 +183,14 @@ public:
 				&& GetNumShapes()		== o.GetNumShapes();
 		for (size_t i = 0; i < GetNumPins() && bOK; i++)
 		{
-			bOK =  m_nodeIdPins[i]	== o.m_nodeIdPins[i]
-				&& m_origIdPins1[i]	== o.m_origIdPins1[i]
-				&& m_origIdPins2[i] == o.m_origIdPins2[i]
-				&& m_pinLabels[i]	== o.m_pinLabels[i]
-				&& m_pinAligns[i]	== o.m_pinAligns[i];
+			bOK =  m_nodeIdPins[i]		== o.m_nodeIdPins[i]
+				&& m_origIdPins1[i]		== o.m_origIdPins1[i]
+				&& m_origIdPins2[i]		== o.m_origIdPins2[i]
+				&& m_layerPrefs[i]		== o.m_layerPrefs[i]
+				&& m_pinOffsetRow[i]	== o.m_pinOffsetRow[i]
+				&& m_pinOffsetCol[i]	== o.m_pinOffsetCol[i]
+				&& m_pinLabels[i]		== o.m_pinLabels[i]
+				&& m_pinAligns[i]		== o.m_pinAligns[i];
 		}
 		for (size_t i = 0; i < GetNumShapes() && bOK; i++)
 		{
@@ -184,6 +205,9 @@ public:
 		m_nodeIdPins.clear();
 		m_origIdPins1.clear();
 		m_origIdPins2.clear();
+		m_layerPrefs.clear();
+		m_pinOffsetRow.clear();
+		m_pinOffsetCol.clear();
 		m_pinLabels.clear();
 		m_pinAligns.clear();
 		m_shapes.clear();
@@ -210,6 +234,18 @@ public:
 			if ( iPinIndex < m_origIdPins2.size() ) m_origIdPins2[iPinIndex] = i;
 		}
 	}
+	void SetLayerPref(const size_t& iPinIndex, const uchar& iPref)
+	{
+		if ( iPinIndex < m_layerPrefs.size() ) m_layerPrefs[iPinIndex] = iPref;
+	}
+	void SetPinOffsetRow(const size_t& iPinIndex, const int& i)
+	{
+		if ( iPinIndex < m_pinOffsetRow.size() ) m_pinOffsetRow[iPinIndex] = std::max(-MAX_PAD_OFFSET_MIL, std::min(MAX_PAD_OFFSET_MIL, i));
+	}
+	void SetPinOffsetCol(const size_t& iPinIndex, const int& i)
+	{
+		if ( iPinIndex < m_pinOffsetCol.size() ) m_pinOffsetCol[iPinIndex] = std::max(-MAX_PAD_OFFSET_MIL, std::min(MAX_PAD_OFFSET_MIL, i));
+	}
 	void SetPinLabel(const size_t& iPinIndex, const std::string& s)
 	{
 		if ( iPinIndex < m_pinLabels.size() ) m_pinLabels[iPinIndex] = s;
@@ -239,6 +275,9 @@ public:
 		m_nodeIdPins.clear();	m_nodeIdPins.resize(numPins, BAD_NODEID);
 		m_origIdPins1.clear();	m_origIdPins1.resize(numPins, BAD_NODEID);
 		m_origIdPins2.clear();	m_origIdPins2.resize(numPins, BAD_NODEID);
+		m_layerPrefs.clear();	m_layerPrefs.resize(numPins, LAYER_X);
+		m_pinOffsetRow.clear();	m_pinOffsetRow.resize(numPins, 0);
+		m_pinOffsetCol.clear();	m_pinOffsetCol.resize(numPins, 0);
 		m_pinLabels.clear();	m_pinLabels.resize(numPins, "");
 		m_pinAligns.clear();	m_pinAligns.resize(numPins, Qt::AlignHCenter);
 		SetDefaultPinLabels();
@@ -286,6 +325,21 @@ public:
 		else
 			return ( iPinIndex < m_origIdPins2.size() ) ? m_origIdPins2[iPinIndex] : badNodeId;
 	}
+	const uchar&		GetLayerPref(const size_t& iPinIndex) const
+	{
+		static uchar noPref(LAYER_X);
+		return ( iPinIndex < m_layerPrefs.size() ) ? m_layerPrefs[iPinIndex] : noPref;
+	}
+	const int&			GetPinOffsetRow(const size_t& iPinIndex) const
+	{
+		static int defaultOffset(0);
+		return ( iPinIndex < m_pinOffsetRow.size() ) ? m_pinOffsetRow[iPinIndex] : defaultOffset;
+	}
+	const int&			GetPinOffsetCol(const size_t& iPinIndex) const
+	{
+		static int defaultOffset(0);
+		return ( iPinIndex < m_pinOffsetCol.size() ) ? m_pinOffsetCol[iPinIndex] : defaultOffset;
+	}
 	const std::string&	GetPinLabel(const size_t& iPinIndex) const
 	{
 		static const std::string emptyStr("");
@@ -324,17 +378,29 @@ public:
 										  else		SetPinFlags( m_iPinFlags & ~PIN_CUSTOM ); }
 	bool GetCustomPads() const			{ return ( GetPinFlags() & PIN_CUSTOM ) != 0; }
 
-	void GetSafeBounds(double& L, double& R, double& T, double& B) const
+	void GetSafeBounds(double& L, double& R, double& T, double& B, bool bFill = true) const
 	{
-		L = T =  DBL_MAX;
-		R = B = -DBL_MAX;
-		double l,r,t,b;
+		// First consider footprint bounds (for direction 'W')
+		const double dW( 0.5 * GetCols() ), dH( 0.5 * GetRows() );
+		L = -dW;	R = dW;		T = -dH;	B = dH;
+		// Then consider the list of shapes (for direction 'W')
+		double l,r,t,b;	// Working variables
 		for (auto& o : m_shapes)
 		{
-			o.GetSafeBounds(l,r,t,b);
+			if ( !bFill && o.GetDrawFill() ) continue;	// If view is not drawing filled shapes, skip them
+			o.GetSafeBounds(l, r, t, b);
 			L = std::min(L,l);	T = std::min(T,t);
 			R = std::max(R,r);	B = std::max(B,b);
 		}
+		// Handle other component directions
+		switch ( GetDirection() )
+		{
+			case 'E':	l = -R;	t = -B;	r = -L;	b = -T;	break;
+			case 'N':	l = -B;	t =  L;	r = -T;	b =  R;	break;
+			case 'S':	l =  T;	t = -R;	r =  B;	b = -L;	break;
+			default:	return;
+		}
+		L = l;	T = t;	R = r;	B = b;
 	}
 	std::string GetFullTypeStr() const		// For SIP/DIP types, append the number of pins
 	{
@@ -365,6 +431,32 @@ public:
 	const int&			GetCompCols() const	{ return GetCols( GetDirection() ); }
 	int					GetLastRow() const	{ return GetRow() + GetCompRows() - 1; }
 	int					GetLastCol() const	{ return GetCol() + GetCompCols() - 1; }
+	void				GetCompPinOffsets(const size_t& iPinIndex, int& Xmil, int& Ymil) const
+	{
+		switch ( GetDirection() )
+		{
+			case 'E':	Xmil = -GetPinOffsetCol(iPinIndex);	Ymil = -GetPinOffsetRow(iPinIndex); return;
+			case 'N':	Xmil = -GetPinOffsetRow(iPinIndex);	Ymil =  GetPinOffsetCol(iPinIndex); return;
+			case 'S':	Xmil =  GetPinOffsetRow(iPinIndex);	Ymil = -GetPinOffsetCol(iPinIndex); return;
+			default:	Xmil =  GetPinOffsetCol(iPinIndex);	Ymil =  GetPinOffsetRow(iPinIndex); return;
+		}
+	}
+	void				SetCompPinOffsets(const size_t& iPinIndex, const int& Xmil, const int& Ymil)
+	{
+		switch ( GetDirection() )
+		{
+			case 'E':	SetPinOffsetCol(iPinIndex, -Xmil);	SetPinOffsetRow(iPinIndex, -Ymil); return;
+			case 'N':	SetPinOffsetRow(iPinIndex, -Xmil);	SetPinOffsetCol(iPinIndex,  Ymil); return;
+			case 'S':	SetPinOffsetRow(iPinIndex,  Xmil);	SetPinOffsetCol(iPinIndex, -Ymil); return;
+			default:	SetPinOffsetCol(iPinIndex,  Xmil);	SetPinOffsetRow(iPinIndex,  Ymil); return;
+		}
+	}
+	void				IncCompPinOffsets(const size_t& iPinIndex, const int& dX, const int& dY)
+	{
+			int Xmil, Ymil;
+			GetCompPinOffsets(iPinIndex, Xmil, Ymil);
+			SetCompPinOffsets(iPinIndex, Xmil + dX, Ymil + dY);
+	}
 	const CompElement*	GetCompElement(const int& compRow, const int& compCol) const
 	{
 		return FootPrint::Get(0, compRow, compCol, (char)GetDirection());
@@ -458,6 +550,9 @@ public:
 	{
 		switch( GetType() )
 		{
+			case COMP::INVALID:
+			case COMP::VERO_NUMBER:
+			case COMP::VERO_LETTER:
 			case COMP::MARK:
 			case COMP::WIRE:
 			case COMP::TRACKS:	return false;
@@ -571,6 +666,13 @@ public:
 			inStream.Load(m_origIdPins1[i]);
 			if ( inStream.GetVersion() >= VRT_VERSION_34 )
 				inStream.Load(m_origIdPins2[i]);	// Added in VRT_VERSION_34
+			if ( inStream.GetVersion() >= VRT_VERSION_45 )
+				inStream.Load(m_layerPrefs[i]);		// Added in VRT_VERSION_45
+			if ( inStream.GetVersion() >= VRT_VERSION_46 )
+			{
+				inStream.Load(m_pinOffsetRow[i]);	// Added in VRT_VERSION_46
+				inStream.Load(m_pinOffsetCol[i]);	// Added in VRT_VERSION_46
+			}
 			if ( inStream.GetVersion() >= VRT_VERSION_7 )
 				inStream.Load(m_pinLabels[i]);		// Added in VRT_VERSION_7
 			if ( inStream.GetVersion() >= VRT_VERSION_30 )
@@ -617,6 +719,9 @@ public:
 			outStream.Save(m_nodeIdPins[i]);
 			outStream.Save(m_origIdPins1[i]);
 			outStream.Save(m_origIdPins2[i]);	// Added in VRT_VERSION_34
+			outStream.Save(m_layerPrefs[i]);	// Added in VRT_VERSION_45
+			outStream.Save(m_pinOffsetRow[i]);	// Added in VRT_VERSION_46
+			outStream.Save(m_pinOffsetCol[i]);	// Added in VRT_VERSION_46
 			outStream.Save(m_pinLabels[i]);		// Added in VRT_VERSION_7
 			outStream.Save(m_pinAligns[i]);		// Added in VRT_VERSION_30
 		}
@@ -635,6 +740,9 @@ private:
 	std::vector<int>			m_nodeIdPins;		// NodeIds of the pins
 	std::vector<int>			m_origIdPins1;		// NodeIds under the pins BEFORE placement (1st layer)
 	std::vector<int>			m_origIdPins2;		// NodeIds under the pins BEFORE placement (2nd Layer)
+	std::vector<uchar>			m_layerPrefs;		// Prefered layers of the pins
+	std::vector<int>			m_pinOffsetRow;		// Pin row offset (-50 mil to +50 mil) to allow pad shifts in PCB mode
+	std::vector<int>			m_pinOffsetCol;		// Pin row offset (-50 mil to +50 mil) to allow pad shifts in PCB mode
 	std::vector<std::string>	m_pinLabels;		// Pin labels
 	std::vector<int>			m_pinAligns;		// Pin label alignments (Qt::AlignLeft,Qt::AlignRight,Qt::AlignHCenter)
 	std::vector<Shape>			m_shapes;			// For rendering components. Coordinates are RELATIVE to footprint centre.
