@@ -34,10 +34,10 @@
 // each Element knows what it can be connected to without having
 // to go through the parent Grid object.
 
-const int			TRAX_COMPID = -2;		// The component manager member m_trax has this ID
-const int			BAD_COMPID  = -1;		// Invalid component ID
-const unsigned int	BAD_ROUTEID = UINT_MAX;	// Invalid route (i.e. track section) ID
-const unsigned int	BAD_MH		= UINT_MAX;	// "Infinite" MH distance
+static const int			TRAX_COMPID	= -2;		// The component manager member m_trax has this ID
+static const int			BAD_COMPID	= -1;		// Invalid component ID
+static const unsigned int	BAD_ROUTEID	= UINT_MAX;	// Invalid route (i.e. track section) ID
+static const unsigned int	BAD_MH		= UINT_MAX;	// "Infinite" MH distance
 
 class Element;
 
@@ -261,13 +261,13 @@ public:
 		ToggleCodeBit(Opposite(iNbr), m_pNbr[iNbr]->m_iCode);	// Keep consistent with nbr
 		// Toggles are only done by the user so set the flag accordingly
 		Element* pNbr = GetNbr(iNbr);
-		ClearFlagBits(AUTOSET|VEROSET);			SetFlagBits(USERSET);
-		pNbr->ClearFlagBits(AUTOSET|VEROSET);	pNbr->SetFlagBits(USERSET);
+		WipeFlagBits(AUTOSET|VEROSET);			MarkFlagBits(USERSET);
+		pNbr->WipeFlagBits(AUTOSET|VEROSET);	pNbr->MarkFlagBits(USERSET);
 		// Handle wire ends
-		Element* pW = GetW(0);	if ( pW ) { pW->ClearFlagBits(AUTOSET|VEROSET);	pW->SetFlagBits(USERSET); }
-		pW = GetW(1);			if ( pW ) { pW->ClearFlagBits(AUTOSET|VEROSET);	pW->SetFlagBits(USERSET); }
-		pW = pNbr->GetW(0);		if ( pW ) { pW->ClearFlagBits(AUTOSET|VEROSET);	pW->SetFlagBits(USERSET); }
-		pW = pNbr->GetW(1);		if ( pW ) { pW->ClearFlagBits(AUTOSET|VEROSET);	pW->SetFlagBits(USERSET); }
+		Element* pW = GetW(0);	if ( pW ) { pW->WipeFlagBits(AUTOSET|VEROSET);	pW->MarkFlagBits(USERSET); }
+		pW = GetW(1);			if ( pW ) { pW->WipeFlagBits(AUTOSET|VEROSET);	pW->MarkFlagBits(USERSET); }
+		pW = pNbr->GetW(0);		if ( pW ) { pW->WipeFlagBits(AUTOSET|VEROSET);	pW->MarkFlagBits(USERSET); }
+		pW = pNbr->GetW(1);		if ( pW ) { pW->WipeFlagBits(AUTOSET|VEROSET);	pW->MarkFlagBits(USERSET); }
 	}
 	bool SwapDiagLinks()
 	{
