@@ -251,9 +251,42 @@ static const char* szVEROROUTE_VERSION =
 //"1.98";	// Put in extra checks to stop potential crashes arising from a corrupted layout.
 			// Added file menu option to fix a corrupted layout.
 //"1.99";	// Code changes to help prevent layout from becoming corrupted.
-  "2.00";	// Bug fix: Poor-man's 2-layer Gerber export was not handling crossing wires.
+//"2.00";	// Bug fix: Poor-man's 2-layer Gerber export was not handling crossing wires.
 			// Bug fix: Isolated nodes were rendered to Gerber with pad-width instead of track-width.
 			// Bug fix: Isolated nodes were rendered to top layer for poor-man's 2-layer Gerber export.
 			// Improvement: Allow TAB to change focus of dialog controls.
 			// Improvement: Reworked some icons that did not work well with dark themes.
 			// Removed menu item to fix corruption.  Old corrupted VRTS are now auto-fixed on load.
+//"2.01";	// Bug fix: Chains of wires sharing holes sometimes not colored completely.
+			// Added ability to define custom pad/hole size in component editor.
+			// Added ability to add veroboard labels (numbers or letters).
+			// Improved algorithm for solder bridges.  Don't show a solder bridge if a wire is already doing that job.
+//"2.02";	// Bug fix: Fill slider should be enabled when track slider is set to Off.
+			// Bug fix: Calculation of minimum track separation was not correct when using fat tracks style.
+			// Bug fix: Changed render order in Color/Mono mode to fix issues with how custom sizes pads are drawn.
+			// Track width changes:  Minimum allowed track widths lowered to 4 mil.
+			//                       Minimum allowed via hole lowered to 8 mil.
+			//                       Minimum allowed via pad lowered to 16 mil.
+			//                       Maximum custom pad size increased to 130 mil.
+			//                       Maximum custom hole size increased to 100 mil.
+			// New feature:  Add ability to show locations of minimum track separation.
+			// Updated tutorials.
+//"2.03";	// Bug fix: Wire rendering broken in 2.02.
+			// New feature. Can specify layer preference on component pins of 2-layer PCBs.
+			// Updated tutorials.
+//"2.04";	// Bug fix: Since 2.02, VeroRoute freezes if the grid is made 1 column wide.
+			// Omit deprecated G codes from Gerber export, and allow Gerber export in mm or inches.
+			// Allow circuit panning/grid resize by grabbing the grid with the right mouse button (ctrl key optional).
+			// Improve GUI behaviour in Tutorials by passing key presses from the Info dialog to the main window.
+//"2.05";	// Bug fix: Code for filling parts with color not correctly handling shape transformations.
+			// Bug fix: Track separation info in the Rendering dialog not updating when auto-routing toggled.
+			// Bug fix: Components with lines outside the footprint area could be rendered off screen
+			// leading to an invalid set of Gerber files (e.g. GTO file having points outside the boundary in the GKO file).
+			// Improvement: Total rewrite of minimum track separation for improved accuracy.
+			// Improvement: Show points for all layers when displaying locations of minimum track separation.
+			// New feature: In PCB mode, individual pad locations can be shifted by up to 50 mil using Ctrl + Arrow keys.
+  "2.06";	// Bug fix: In Rendering Options, the "Show closest tracks" box was not being hidden when minimum separation was same as Gap size.
+			// Bug fix. In Fat Tracks mode with diagonals, tracks in the view were not all of correct width, but the ones in Gerber export were.
+			// Bug fix: In Fat Tracks mode with diagonals, the track separation algorithm was not handling all cases properly.
+			// Bug fix: In Fat Tracks mode, Gerber export should create an aperture for pads even if there are no parts in the circuit.
+			// Improvement: Allow reset pin layer preference ("T" + right button) while moving mouse instead of having to do single clicks.
