@@ -62,7 +62,7 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	const std::string iconStr = m_tutorialsPathStr + "/veroroute.png";
 	setWindowIcon(QIcon( iconStr.c_str() ));
 
-	// Create the scrollable area and make it occuoy the main window area
+	// Create the scrollable area and make it occupy the main window area
 	m_label			= new QLabel(this);
 	m_scrollArea	= new MyScrollArea(this);
 	m_label->setBackgroundRole(QPalette::Base);
@@ -114,7 +114,7 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	m_blackPen			= QPen(Qt::black, 0,				Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 	m_whitePen			= QPen(Qt::white, 0,				Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 	m_redPen			= QPen(Qt::red, 0,					Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
-	m_findPen			= QPen(QColor(255,128,0,255), 0,	Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
+	m_orangePen			= QPen(QColor(255,128,0,255), 0,	Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 	m_lightBluePen		= QPen(QColor(96, 96, 255, 255), 0,	Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 	m_varPen			= QPen(Qt::black, 0,				Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 	m_dotPen			= QPen(QColor(96,96,96,255), 0,		Qt::DotLine,   Qt::RoundCap, Qt::RoundJoin);
@@ -132,8 +132,10 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	QObject::connect(ui->actionImportOrcad,				SIGNAL(triggered()), this, SLOT(ImportOrcad()));
 	QObject::connect(ui->actionWrite_PDF,				SIGNAL(triggered()), this, SLOT(WritePDF()));
 	QObject::connect(ui->actionWrite_PNG,				SIGNAL(triggered()), this, SLOT(WritePNG()));
-	QObject::connect(ui->actionWrite_Gerber,			SIGNAL(triggered()), this, SLOT(WriteGerber()));
-	QObject::connect(ui->actionWrite_Gerber2,			SIGNAL(triggered()), this, SLOT(WriteGerber2()));
+	QObject::connect(ui->actionWrite_Gerber1_in,		SIGNAL(triggered()), this, SLOT(WriteGerber1in()));
+	QObject::connect(ui->actionWrite_Gerber1_mm,		SIGNAL(triggered()), this, SLOT(WriteGerber1mm()));
+	QObject::connect(ui->actionWrite_Gerber2_in,		SIGNAL(triggered()), this, SLOT(WriteGerber2in()));
+	QObject::connect(ui->actionWrite_Gerber2_mm,		SIGNAL(triggered()), this, SLOT(WriteGerber2mm()));
 	QObject::connect(ui->actionClearRecent,				SIGNAL(triggered()), this, SLOT(ClearRecentFiles()));
 	QObject::connect(ui->actionQuit,					SIGNAL(triggered()), this, SLOT(Quit()));
 	QObject::connect(ui->actionZoom_In,					SIGNAL(triggered()), this, SLOT(ZoomIn()));
@@ -202,6 +204,8 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	QObject::connect(ui->actionSwitchDT,				SIGNAL(triggered()), this, SLOT(AddSwitchDT()));
 	QObject::connect(ui->actionSwitchST_DIP,			SIGNAL(triggered()), this, SLOT(AddSwitchST_DIP()));
 	QObject::connect(ui->actionTextBox,					SIGNAL(triggered()), this, SLOT(AddTextBox()));
+	QObject::connect(ui->actionVeroNumbers,				SIGNAL(triggered()), this, SLOT(AddVeroNumbers()));
+	QObject::connect(ui->actionVeroLetters,				SIGNAL(triggered()), this, SLOT(AddVeroLetters()));
 	QObject::connect(ui->actionRenderingDlg,			SIGNAL(triggered()), this, SLOT(ShowRenderingDialog()));
 	QObject::connect(ui->actionWireDlg,					SIGNAL(triggered()), this, SLOT(ShowWireDialog()));
 	QObject::connect(ui->actionControlDlg,				SIGNAL(triggered()), this, SLOT(ShowControlDialog()));
@@ -339,8 +343,8 @@ void MainWindow::ResetView(bool bTutorial)
 {
 	m_mousePos = QPoint(0,0);
 	m_bMouseClick	= m_bLeftClick	= m_bRightClick = m_bCtrlKeyDown  = m_bShiftKeyDown	=false;
-	m_bPaintPins	= m_bPaintBoard	= m_bPaintFlood = m_bDefiningRect = m_bResizingText	= m_bWritePDF = m_bWriteGerber = m_bTwoLayerGerber = false;
-	m_XGRIDOFFSET	= m_YGRIDOFFSET	= 0;
+	m_bPaintPins	= m_bPaintBoard	= m_bPaintFlood = m_bPaintLyrPref = m_bDefiningRect = m_bResizingText = m_bWritePDF = m_bWriteGerber = m_bTwoLayerGerber = false;
+	m_XGRIDOFFSET	= m_YGRIDOFFSET	= m_XCORRECTION = m_YCORRECTION = 0;
 
 	// Try to set m_gridRow, m_gridCol to match the current NodeId in the board
 	m_gridRow = m_gridCol = 0;
@@ -393,7 +397,8 @@ void MainWindow::HandleRouting()
 		m_board.Route(true);	// true ==> Use minimal routing
 		releaseMouse();
 	}
-	m_board.UpdateVias();
+	m_board.UpdateVias();	// Needed before m_renderingDlg->UpdateControls() for via info and track separation
+	m_renderingDlg->UpdateControls();
 	if ( GetCurrentNodeId() != BAD_NODEID )
 	{
 		grabMouse(Qt::WaitCursor);
@@ -674,7 +679,7 @@ void MainWindow::WritePDF()
 	}
 }
 
-void MainWindow::WriteGerber(const bool& bTwoLayerGerber)
+void MainWindow::WriteGerber(const bool& bTwoLayerGerber, const bool& bMetric)
 {
 	m_bTwoLayerGerber = bTwoLayerGerber;
 	m_board.SetHoleType(m_bTwoLayerGerber ? HOLETYPE::PTH : HOLETYPE::NPTH);
@@ -695,7 +700,7 @@ void MainWindow::WriteGerber(const bool& bTwoLayerGerber)
 		const bool bWireVias	= m_bTwoLayerGerber && m_board.GetLyrs() == 1 && m_board.GetCompMgr().GetHavePlacedWires();
 		const bool bVias		= m_board.GetHasVias() || bWireVias;
 
-		if ( m_gWriter.Open(m_gerberFileName.toStdString().c_str(), m_board, bVias, m_bTwoLayerGerber) )
+		if ( m_gWriter.Open(m_gerberFileName.toStdString().c_str(), m_board, bVias, m_bTwoLayerGerber, bMetric) )
 		{
 			const int origlayer = m_board.GetCurrentLayer();
 			for (int lyr = 0, lyrs = m_board.GetLyrs(); lyr < lyrs; lyr++)
@@ -1189,6 +1194,7 @@ void MainWindow::SetCompPadWidth(const int& i)
 	Component& comp = m_board.GetUserComponent();
 	if ( i == comp.GetPadWidth() ) return;	// No change
 	comp.SetPadWidth(i);
+	if ( comp.GetHoleWidth() > i-8 ) comp.SetHoleWidth( i-8 );	// 8 ==> minimum annular ring = 4 mil
 	UpdateHistory("Change custom pad size");
 	UpdateControls();
 	RepaintWithListNodes();
@@ -1200,6 +1206,7 @@ void MainWindow::SetCompHoleWidth(const int& i)
 	Component& comp = m_board.GetUserComponent();
 	if ( i == comp.GetHoleWidth() ) return;	// No change
 	comp.SetHoleWidth(i);
+	if ( comp.GetPadWidth() < i+8 ) comp.SetPadWidth( i+8 );	// 8 ==> minimum annular ring = 4 mil
 	UpdateHistory("Change i hole size");
 	UpdateControls();
 	RepaintWithListNodes();
@@ -1448,6 +1455,8 @@ void MainWindow::SetDiagonalsMax(bool b)
 	const bool bListNodes = ( m_board.GetDiagsMode() == DIAGSMODE::OFF );	// Only ListNodes() again if necessary
 	if ( b && m_board.SetDiagsMode(DIAGSMODE::MAX) )	{ UpdateHistory("Diagonals max"); UpdateControls(); DestroyPixmapCache(); if ( bListNodes ) RepaintWithListNodes(); else RepaintWithRouting(); }
 }
+
+// Rendering options
 void MainWindow::SetPadWidth(int i)			{ if ( m_board.SetPAD_MIL(i)   ) { UpdateHistory("Pad width change");				UpdateControls();	DestroyPixmapCache();	RepaintSkipRouting(); } }
 void MainWindow::SetTrackWidth(int i)		{ if ( m_board.SetTRACK_MIL(i) ) { UpdateHistory("Track width change");				UpdateControls();	DestroyPixmapCache();	RepaintSkipRouting(); } }
 void MainWindow::SetHoleWidth(int i)		{ if ( m_board.SetHOLE_MIL(i)  ) { UpdateHistory("Hole width change");				UpdateControls();	RepaintSkipRouting(); } }
@@ -1456,14 +1465,13 @@ void MainWindow::SetMaskWidth(int i)		{ if ( m_board.SetMASK_MIL(i)  ) { UpdateH
 void MainWindow::SetSilkWidth(int i)		{ if ( m_board.SetSILK_MIL(i)  ) { UpdateHistory("Silkscreen line width change");	UpdateControls();	RepaintSkipRouting(); } }
 void MainWindow::SetEdgeWidth(int i)		{ if ( m_board.SetEDGE_MIL(i)  ) { UpdateHistory("Board edge margin change");		UpdateControls();	RepaintSkipRouting(); } }
 void MainWindow::SetViaPadWidth(int i)		{ if ( m_board.SetVIAPAD_MIL(i)) { UpdateHistory("Via pad width change");			UpdateControls();	DestroyPixmapCache();	RepaintSkipRouting(); } }
-void MainWindow::SetViaHoleWidth(int i)		{ if ( m_board.SetVIAHOLE_MIL(i)){ UpdateHistory("Via hole width change");			UpdateControls();	RepaintSkipRouting(); } }
-
-// Rendering dialog
+void MainWindow::SetViaHoleWidth(int i)		{ if ( m_board.SetVIAHOLE_MIL(i)){ UpdateHistory("Via hole width change");			UpdateControls();	DestroyPixmapCache();	RepaintSkipRouting(); } }
 void MainWindow::SetTextSizeComp(int i)		{ if ( m_board.SetTextSizeComp(i) )		  { UpdateHistory("Text size change (component)");	RepaintSkipRouting(); } }
 void MainWindow::SetTextSizePins(int i)		{ if ( m_board.SetTextSizePins(i) )		  { UpdateHistory("Text size change (pins)");		RepaintSkipRouting(); } }
 void MainWindow::SetTargetRows(int i)		{ if ( m_board.SetTargetRows(i) )		  { UpdateHistory("Target board height change");	RepaintSkipRouting(); } }
 void MainWindow::SetTargetCols(int i)		{ if ( m_board.SetTargetCols(i) )		  { UpdateHistory("Target board width change");		RepaintSkipRouting(); } }
 void MainWindow::SetShowTarget(bool b)		{ if ( m_board.SetShowTarget(b) )		  { UpdateHistory("Target board on/off");			RepaintSkipRouting(); } }
+void MainWindow::SetShowCloseTracks(bool b)	{ if ( m_board.SetShowCloseTracks(b) )	  { UpdateHistory("Show closest tracks on/off");	UpdateControls();	RepaintSkipRouting(); } }
 void MainWindow::SetAntialiasOff(bool b)	{ if ( b && m_board.SetRenderQuality(0) ) { UpdateHistory("Anti-alias off");  DestroyPixmapCache(); RepaintSkipRouting(); } }
 void MainWindow::SetAntialiasOn(bool b)		{ if ( b && m_board.SetRenderQuality(1) ) { UpdateHistory("Anti-alias on");   DestroyPixmapCache(); RepaintSkipRouting(); } }
 void MainWindow::SetAntialiasHigh(bool b)	{ if ( b && m_board.SetRenderQuality(2) ) { UpdateHistory("Anti-alias high"); DestroyPixmapCache(); RepaintSkipRouting(); } }
@@ -1523,6 +1531,20 @@ void MainWindow::DefinerToggledPinLabels(bool b)
 		EnableCompDialogControls();
 	}
 }
+void MainWindow::DefinerToggledCustomFlag(bool b)
+{
+	CompDefiner& compDefiner = GetCompDefiner();
+	bool bChanged(false);
+	if ( b )
+		bChanged = compDefiner.SetPinFlags( compDefiner.GetPinFlags() | PIN_CUSTOM );
+	else
+		bChanged = compDefiner.SetPinFlags( compDefiner.GetPinFlags() & ~PIN_CUSTOM );
+	if ( bChanged )
+	{
+		UpdateHistory("Toggle custom pad/hole size");
+		EnableCompDialogControls();
+	}
+}
 void MainWindow::DefinerToggleShapeLine(bool b)
 {
 	const bool bChanged = GetCompDefiner().SetLine(b);
@@ -1543,17 +1565,19 @@ void MainWindow::DefinerToggleShapeFill(bool b)
 		RepaintSkipRouting();
 	}
 }
-void MainWindow::DefinerWidthChanged(int i)	{ if ( GetCompDefiner().SetWidth(i)		) { UpdateHistory("Action"); EnableCompDialogControls(); UpdateCompDialog(); RepaintSkipRouting(); } }
-void MainWindow::DefinerHeightChanged(int i){ if ( GetCompDefiner().SetHeight(i)	) { UpdateHistory("Action"); EnableCompDialogControls(); UpdateCompDialog(); RepaintSkipRouting(); } }
-void MainWindow::DefinerSetPinNumber(int i)	{ if ( GetCompDefiner().SetPinNumber(i)	) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
-void MainWindow::DefinerIncPinNumber(bool b){ if ( GetCompDefiner().IncPinNumber(b)	) { UpdateHistory("Action"); UpdateCompDialog();		 RepaintSkipRouting(); } }	// Called using mouse wheel in view
-void MainWindow::DefinerSetCX(double d)		{ if ( GetCompDefiner().SetCX(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
-void MainWindow::DefinerSetCY(double d)		{ if ( GetCompDefiner().SetCY(-d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }	// Control assumes CY goes up
-void MainWindow::DefinerSetDX(double d)		{ if ( GetCompDefiner().SetDX(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
-void MainWindow::DefinerSetDY(double d)		{ if ( GetCompDefiner().SetDY(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
-void MainWindow::DefinerSetA1(double d)		{ if ( GetCompDefiner().SetA1(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
-void MainWindow::DefinerSetA2(double d)		{ if ( GetCompDefiner().SetA2(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
-void MainWindow::DefinerSetA3(double d)		{ if ( GetCompDefiner().SetA3(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
+void MainWindow::DefinerWidthChanged(int i)		{ if ( GetCompDefiner().SetWidth(i)		) { UpdateHistory("Action"); EnableCompDialogControls(); UpdateCompDialog(); RepaintSkipRouting(); } }
+void MainWindow::DefinerHeightChanged(int i)	{ if ( GetCompDefiner().SetHeight(i)	) { UpdateHistory("Action"); EnableCompDialogControls(); UpdateCompDialog(); RepaintSkipRouting(); } }
+void MainWindow::DefinerPadWidthChanged(int i)	{ if ( GetCompDefiner().SetPadWidth(i)	) { UpdateHistory("Action"); EnableCompDialogControls(); UpdateCompDialog(); RepaintSkipRouting(); } }
+void MainWindow::DefinerHoleWidthChanged(int i)	{ if ( GetCompDefiner().SetHoleWidth(i)	) { UpdateHistory("Action"); EnableCompDialogControls(); UpdateCompDialog(); RepaintSkipRouting(); } }
+void MainWindow::DefinerSetPinNumber(int i)		{ if ( GetCompDefiner().SetPinNumber(i)	) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
+void MainWindow::DefinerIncPinNumber(bool b)	{ if ( GetCompDefiner().IncPinNumber(b)	) { UpdateHistory("Action"); UpdateCompDialog();		 RepaintSkipRouting(); } }	// Called using mouse wheel in view
+void MainWindow::DefinerSetCX(double d)			{ if ( GetCompDefiner().SetCX(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
+void MainWindow::DefinerSetCY(double d)			{ if ( GetCompDefiner().SetCY(-d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }	// Control assumes CY goes up
+void MainWindow::DefinerSetDX(double d)			{ if ( GetCompDefiner().SetDX(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
+void MainWindow::DefinerSetDY(double d)			{ if ( GetCompDefiner().SetDY(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
+void MainWindow::DefinerSetA1(double d)			{ if ( GetCompDefiner().SetA1(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
+void MainWindow::DefinerSetA2(double d)			{ if ( GetCompDefiner().SetA2(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
+void MainWindow::DefinerSetA3(double d)			{ if ( GetCompDefiner().SetA3(d)		) { UpdateHistory("Action"); EnableCompDialogControls(); RepaintSkipRouting(); } }
 void MainWindow::DefinerBuild()
 {
 	assert( GetCompDefiner().GetIsValid() );
@@ -1699,8 +1723,8 @@ void MainWindow::UpdateControls()
 	const bool		bThin			= !m_board.GetVeroTracks() && !m_board.GetCurvedTracks() && !m_board.GetFatTracks();
 	const bool		bCurved			= !m_board.GetVeroTracks() &&  m_board.GetCurvedTracks();
 
-	ui->actionWrite_Gerber->setEnabled( bPCB && !bCompEdit && !m_board.GetMirrored() && !m_board.GetVeroTracks() && m_board.GetLyrs() == 1);
-	ui->actionWrite_Gerber2->setEnabled(bPCB && !bCompEdit && !m_board.GetMirrored() && !m_board.GetVeroTracks());
+	ui->menuExport_as_Gerber_1_Layer->setEnabled(bPCB && !bCompEdit && !m_board.GetMirrored() && !m_board.GetVeroTracks() && m_board.GetLyrs() == 1);
+	ui->menuExport_as_Gerber_2_Layer->setEnabled(bPCB && !bCompEdit && !m_board.GetMirrored() && !m_board.GetVeroTracks());
 	ui->actionMerge->setEnabled(    !bPCB && !bCompEdit);
 	ui->actionWrite_PDF->setEnabled(!bPCB && !bCompEdit);
 	ui->menuAdd->setEnabled( !bCompEdit && m_board.GetCompMode() != COMPSMODE::OFF && !m_board.GetMirrored() );
@@ -1717,21 +1741,21 @@ void MainWindow::UpdateControls()
 
 	ui->actionCopy->setEnabled( bTextOK || bCompOK );
 	if ( bTextOK )	// Text Box takes precedence over comps
-		ui->actionCopy->setText("Copy+Paste selected Text Box");
+		ui->actionCopy->setText( QString("Copy+Paste selected Text Box") );
 	else if ( bCompOK )
-		ui->actionCopy->setText("Copy+Paste selected Part(s)");
+		ui->actionCopy->setText( QString("Copy+Paste selected Part(s)") );
 	else
-		ui->actionCopy->setText("Copy+Paste selected Part(s) / Text Box");
+		ui->actionCopy->setText( QString(" Copy+Paste selected Part(s) / Text Box") );
 	ui->actionGroup->setEnabled( bCompOK && groupMgr.CanGroup() );
 	ui->actionUngroup->setEnabled( bCompOK && groupMgr.CanUnGroup() );
 	ui->actionSelectAll->setEnabled( bCompActionsOK && !compMgr.GetMapIdToComp().empty() );
 	ui->actionDelete->setEnabled( bTextOK || bCompOK );
 	if ( bTextOK )	// Text Box takes precedence over comps
-		ui->actionDelete->setText("Delete selected Text Box");
+		ui->actionDelete->setText( QString("Delete selected Text Box") );
 	else if ( bCompOK )
-		ui->actionDelete->setText("Delete selected Part(s)");
+		ui->actionDelete->setText( QString("Delete selected Part(s)") );
 	else
-		ui->actionDelete->setText("Delete selected Part(s) / Text Box");
+		ui->actionDelete->setText( QString("Delete selected Part(s) / Text Box") );
 
 	ui->actionCrop->setEnabled( !bCompEdit );
 	ui->actionTextBox->setEnabled( !bCompEdit && !bPCB && m_board.GetShowText() );
@@ -1743,6 +1767,7 @@ void MainWindow::UpdateControls()
 	ui->actionToggleFlipV->setEnabled( !bCompEdit );
 	const bool bPinLabels = m_board.GetCompMode() != COMPSMODE::OFF && ( bNoTracks || bColor );
 	ui->actionTogglePinLabels->setEnabled( bPinLabels );
+
 	ui->actionToggleGrid->setChecked( m_board.GetShowGrid() && !bPCB );
 	ui->actionToggleText->setChecked( m_board.GetShowText() && !bCompEdit && !bPCB );
 	ui->actionToggleFlipH->setChecked( m_board.GetFlipH()   && !bCompEdit );
@@ -1785,7 +1810,7 @@ void MainWindow::UpdateControls()
 	UpdateTextDialog(true);	// true ==> full
 
 	m_pinDlg->Update();
-	m_board.UpdateVias();	// Needed before m_renderingDlg->UpdateControls() for via info
+	m_board.UpdateVias();	// Needed before m_renderingDlg->UpdateControls() for via info and track separation
 	m_renderingDlg->UpdateControls();
 	m_wireDlg->UpdateControls();
 	m_controlDlg->UpdateCompControls();
