@@ -92,6 +92,7 @@ public:
 	const bool&	GetPaintPins() const	{ return m_bPaintPins;		}
 	const bool&	GetPaintBoard() const	{ return m_bPaintBoard;		}
 	const bool&	GetPaintFlood() const	{ return m_bPaintFlood;		}
+	const bool&	GetPaintLyrPref() const	{ return m_bPaintLyrPref;	}
 	const bool&	GetDefiningRect() const	{ return m_bDefiningRect;	}
 	const bool&	GetResizingText() const	{ return m_bResizingText;	}
 	void		SetCtrlKeyDown(bool b)	{ m_bCtrlKeyDown	= b; }
@@ -99,6 +100,7 @@ public:
 	void		SetPaintPins(bool b)	{ m_bPaintPins		= b; }
 	void		SetPaintBoard(bool b)	{ m_bPaintBoard		= b; }
 	void		SetPaintFlood(bool b)	{ m_bPaintFlood		= b; }
+	void		SetPaintLyrPref(bool b)	{ m_bPaintLyrPref	= b; }
 	void		SetDefiningRect(bool b)	{ m_bDefiningRect	= b; }
 	void		SetResizingText(bool b)	{ m_bResizingText	= b; }
 protected:
@@ -128,8 +130,11 @@ public slots:
 	void ImportOrcad();
 	void WritePDF();
 	void WritePNG();
-	void WriteGerber(const bool& bTwoLayerGerber = false);
-	void WriteGerber2() { WriteGerber(true); }
+	void WriteGerber(const bool& bTwoLayerGerber, const bool& bMetric);
+	void WriteGerber1in()	{ WriteGerber(false, false); }
+	void WriteGerber1mm()	{ WriteGerber(false, true); }
+	void WriteGerber2in()	{ WriteGerber(true, false); }
+	void WriteGerber2mm()	{ WriteGerber(true, true); }
 	void ClearRecentFiles();
 	void Quit();
 	// View menu items
@@ -201,6 +206,8 @@ public slots:
 	void AddSwitchST()			{ AddPart(COMP::SWITCH_ST); }
 	void AddSwitchDT()			{ AddPart(COMP::SWITCH_DT); }
 	void AddSwitchST_DIP()		{ AddPart(COMP::SWITCH_ST_DIP); }
+	void AddVeroNumbers()		{ AddPart(COMP::VERO_NUMBER); }
+	void AddVeroLetters()		{ AddPart(COMP::VERO_LETTER); }
 	void AddTextBox()
 	{
 		SetCurrentTextId(BAD_TEXTID);
@@ -294,6 +301,7 @@ public slots:
 	void SetDiagonalsOff(bool b);
 	void SetDiagonalsMin(bool b);
 	void SetDiagonalsMax(bool b);
+	// Rendering options
 	void SetTrackWidth(int i);
 	void SetHoleWidth(int i);
 	void SetPadWidth(int i);
@@ -303,12 +311,12 @@ public slots:
 	void SetEdgeWidth(int i);
 	void SetViaPadWidth(int i);
 	void SetViaHoleWidth(int i);
-	// Rendering options
 	void SetTextSizeComp(int i);
 	void SetTextSizePins(int i);
 	void SetTargetRows(int i);
 	void SetTargetCols(int i);
 	void SetShowTarget(bool b);
+	void SetShowCloseTracks(bool b);
 	void SetAntialiasOff(bool b);
 	void SetAntialiasOn(bool b);
 	void SetAntialiasHigh(bool b);
@@ -342,8 +350,11 @@ public slots:
 	void DefinerSetImportStr(const QString& str);
 	void DefinerWidthChanged(int i);
 	void DefinerHeightChanged(int i);
+	void DefinerPadWidthChanged(int i);
+	void DefinerHoleWidthChanged(int i);
 	void DefinerSetPinShapeType(const QString& str);
 	void DefinerToggledPinLabels(bool b);
+	void DefinerToggledCustomFlag(bool b);
 	void DefinerToggleShapeLine(bool b);
 	void DefinerToggleShapeFill(bool b);
 	void DefinerSetPinNumber(int i);
@@ -371,12 +382,12 @@ private:
 	void DestroyPixmapCache();
 	void CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colorManager);
 	void PaintViaGrey(const GuiControl& guiCtrl, QPainter& painter, const QPointF& pC);
-	void PaintPadGrey(const GuiControl& guiCtrl, QPainter& painter, const QPointF& pC, const int& iPadWidthMIL = 0);
+	void PaintPadGrey(const GuiControl& guiCtrl, QPainter& painter, QPen& pen, const QPointF& pC, const int& iPadWidthMIL = 0);
 	void PaintVia(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, const bool& bGap = false);	// Helper
 	void PaintPad(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, const int& iPadWidthMIL = 0, const int& iHoleWidth_MIL = 0, const bool& bGap = false);	// Helper
 	void PaintTag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const int& iPadWidthMIL, const int& iNbr, const int& iLyr);
 	void PaintDiag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pCorner, bool bLT);
-	void PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const int& iPerimeterCode, const bool bGap = false);	// Helper
+	void PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const QPointF& pCoffset, const int& iPerimeterCode, const bool bHavePad = false, const bool bGap = false);	// Helper
 	void PaintBoard();
 	void PaintCompDefiner();
 	void HandleRouting();
@@ -464,7 +475,7 @@ private:
 	QPen	m_blackPen;
 	QPen	m_whitePen;
 	QPen	m_redPen;
-	QPen	m_findPen;
+	QPen	m_orangePen;
 	QPen	m_lightBluePen;
 	QPen	m_varPen;
 	QPen	m_dotPen;
@@ -530,6 +541,7 @@ private:
 	bool		m_bPaintPins;			// true ==> allow paint the component pins (and the board)
 	bool		m_bPaintBoard;			// true ==> allow paint the board only (not the component pins)
 	bool		m_bPaintFlood;			// true ==> allow flood-fill all connected tracks & pins
+	bool		m_bPaintLyrPref;		// true ==> allow paint layer preference to pins
 	bool		m_bDefiningRect;		// true ==> user is defining a rectangle
 	bool		m_bResizingText;		// true ==> user is resizing a text rectangle
 	bool		m_bWritePDF;			// true ==> draw to PDF file instead of screen
@@ -539,6 +551,8 @@ private:
 	bool		m_bTemplatesDir;		// true ==> have "templates" folder
 	int			m_XGRIDOFFSET;			// So we can centre when writing to PDF
 	int			m_YGRIDOFFSET;			// So we can centre when writing to PDF
+	int			m_XCORRECTION;			// So we can fully render pads larger than 100 mil diameter
+	int			m_YCORRECTION;			// So we can fully render pads larger than 100 mil diameter
 	int			m_gridRow;				// Board row corresponding to mouse position
 	int			m_gridCol;				// Board col correspondong to mouse position
 	int			m_iTutorialNumber = -1;	// Tutorial file number 0,1,2,... (or -1 if not in tutorial mode)
