@@ -20,6 +20,8 @@
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
 
+static const bool ALLOW_SMART_PAN_WITHOUT_CTRLKEY = true;
+
 // Following 2 are to slow down the auto-panning while moving components with the mouse
 static std::chrono::steady_clock::time_point g_lastAutoPanTime;
 static bool g_bHaveAutoPanned = false;
@@ -32,6 +34,12 @@ void MainWindow::GetPixMapXY(const QPoint& currentPoint, int& pixmapX, int& pixm
 	const int iToolbarHeight = ( ui->toolBar->isFloating() || ui->toolBar->isHidden() ) ? 0 : ui->toolBar->height();
 	pixmapX = currentPoint.x() + m_scrollArea->horizontalScrollBar()->value();
 	pixmapY = currentPoint.y() + m_scrollArea->verticalScrollBar()->value() - ui->menuBar->height()- iToolbarHeight;
+
+	int gndL, gndR, gndT, gndB;
+	m_board.GetGroundFillBounds(gndL, gndR, gndT, gndB);
+
+	pixmapX += gndL;
+	pixmapY += gndT;
 }
 
 void MainWindow::GetRowCol(const QPoint& currentPoint, int& row, int& col, double& deltaRow, double& deltaCol) const
@@ -58,9 +66,12 @@ void MainWindow::GetRowCol(const QPoint& currentPoint, const int rows, const int
 
 void MainWindow::wheelEvent(QWheelEvent* event)
 {
+	//TODO QWheelEvent::pos() has been deprecated in newer Qt versions.
+	//	To avoid a build warning, use event->position() instead as follows:
+	//	m_mousePos = QPoint((int)event->position().x(), (int)event->position().y());
 	m_mousePos = event->pos();
-	if ( GetShiftKeyDown() ) return;		// Ignore wheel events while trying to group components
-	const bool bBack = ( event->delta() < 0 );
+	if ( GetShiftKeyDown() ) return;	// Ignore wheel events while trying to group components
+	const bool bBack = ( event->angleDelta().y() < 0 );
 	if ( GetCtrlKeyDown() )
 	{
 		if ( bBack ) ZoomOut(); else ZoomIn();
@@ -196,12 +207,14 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 	// Cursor modification
 	if ( GetCtrlKeyDown() )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
-	else if ( GetPaintPins() || GetPaintBoard() || GetPaintFlood() )
+	else if ( GetPaintPins() || GetPaintBoard() || GetPaintFlood() || GetPaintLyrPref() )
 		centralWidget()->setCursor(Qt::CrossCursor);
 	else if ( GetResizingText() )
 		centralWidget()->setCursor(Qt::SizeFDiagCursor);
 	else if ( GetCurrentTextId() != BAD_TEXTID || GetCurrentCompId() != BAD_COMPID )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
+	else if ( ALLOW_SMART_PAN_WITHOUT_CTRLKEY && m_bRightClick )
+		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else
 		centralWidget()->setCursor(Qt::OpenHandCursor);
 
@@ -211,39 +224,57 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 	if ( GetCurrentTextId() != BAD_TEXTID )
 		return RepaintWithRouting();	// Don't modify nodeId or paint if editing text
 
-	if ( GetPaintPins() || GetPaintBoard() || GetPaintFlood() )
+	if ( GetPaintPins() || GetPaintBoard() || GetPaintFlood() || GetPaintLyrPref() )
 	{
 		if ( m_bLeftClick )		// Paint
 		{
-			if ( GetCurrentNodeId() == BAD_NODEID )	// If trying to left-click paint an BAD_NODEID ...
-				SetCurrentNodeId( m_board.GetNewNodeId() ); // ... use a new NodeId instead
-
-			m_board.GetColorMgr().ReAssignColors();	// Forces colors to be worked out again
-			if ( GetPaintFlood() )
+			if ( GetPaintLyrPref() )
 			{
-				assert( GetCurrentNodeId() != BAD_NODEID );
-				assert( !m_board.GetRoutingEnabled() );	// Sanity check
-
-				const int tmp = GetCurrentNodeId();	// Need to temporarily change current nodeId for HandleRouting()
-				SetCurrentNodeId( m_board.Get(layer, m_gridRow, m_gridCol)->GetNodeId() );
-				HandleRouting();		// Work out MH distances for the flood
-				SetCurrentNodeId(tmp);	// Restore current nodeId
-
-				m_board.FloodNodeId( GetCurrentNodeId() );
-				mouseActionString = "Paint (flood)";
-			}
-			else
-			{
-				const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, GetCurrentNodeId(), GetPaintPins());
+				const bool bChanged = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol, false);	// false ==> toggle
 				if ( !bChanged ) return;
-				mouseActionString = "Paint";
+				mouseActionString = "Toggle pin layer preference";
+			}
+			else 
+			{
+				if ( GetCurrentNodeId() == BAD_NODEID )	// If trying to left-click paint an BAD_NODEID ...
+					SetCurrentNodeId( m_board.GetNewNodeId() ); // ... use a new NodeId instead
+
+				m_board.GetColorMgr().ReAssignColors();	// Forces colors to be worked out again
+				if ( GetPaintFlood() )
+				{
+					assert( GetCurrentNodeId() != BAD_NODEID );
+					assert( !m_board.GetRoutingEnabled() );	// Sanity check
+
+					const int tmp = GetCurrentNodeId();	// Need to temporarily change current nodeId for HandleRouting()
+					SetCurrentNodeId( m_board.Get(layer, m_gridRow, m_gridCol)->GetNodeId() );
+					HandleRouting();		// Work out MH distances for the flood
+					SetCurrentNodeId(tmp);	// Restore current nodeId
+
+					m_board.FloodNodeId( GetCurrentNodeId() );
+					mouseActionString = "Paint (flood)";
+				}
+				else
+				{
+					const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, GetCurrentNodeId(), GetPaintPins());
+					if ( !bChanged ) return;
+					mouseActionString = "Paint";
+				}
 			}
 		}
 		if ( m_bRightClick )	// Unpaint (i.e. erase)
 		{
-			const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, BAD_NODEID, GetPaintPins());
-			if ( !bChanged ) return;
-			mouseActionString = "Erase";
+			if ( GetPaintLyrPref() )
+			{
+				const bool bChanged = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol, true);	// true ==> reset
+				if ( !bChanged ) return;
+				mouseActionString = "Clear pin layer preference";
+			}
+			else
+			{
+				const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, BAD_NODEID, GetPaintPins());
+				if ( !bChanged ) return;
+				mouseActionString = "Erase";
+			}
 		}
 	}
 	else	// Set/Unset current nodeId from board
@@ -304,21 +335,25 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 	const int&			W			= m_board.GetGRIDPIXELS();
 	const int&			layer		= m_board.GetCurrentLayer();
 
-	if ( GetPaintPins() || GetPaintFlood() ) return;// Ignore mouse move while painting pins or flooding
-	if ( GetShiftKeyDown() ) return;				// Ignore mouse move while trying to group components
+	if ( GetPaintPins() || GetPaintFlood() ) return;	// Ignore mouse move while painting pins or flooding
+	if ( GetShiftKeyDown() ) return;										// Ignore mouse move while trying to group components
+
+	bool bSmartPan = GetCtrlKeyDown();
 
 	if ( m_board.GetCompEdit() && GetCurrentShapeId() != BAD_ID && m_bMouseClick )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
+	else if ( bSmartPan )
+		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else if ( GetDefiningRect() || GetResizingText() )
 		centralWidget()->setCursor(Qt::SizeFDiagCursor);
-	else if ( GetCtrlKeyDown() )
-		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else if ( GetPaintBoard() )
 		centralWidget()->setCursor(Qt::CrossCursor);
 	else if ( GetCurrentTextId() != BAD_TEXTID && m_bMouseClick )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else if ( GetCurrentCompId() != BAD_COMPID && m_bMouseClick )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
+	else if ( ALLOW_SMART_PAN_WITHOUT_CTRLKEY && m_bRightClick )
+		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else
 		centralWidget()->setCursor(Qt::OpenHandCursor);
 
@@ -354,28 +389,20 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 			UpdateCompDialog();
 		}
 	}
-	else if ( GetDefiningRect() )
+	else if ( !bSmartPan && GetPaintLyrPref() && m_bRightClick )
+	{
+		const bool bChanged = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol, true);	// true ==> reset
+		if ( !bChanged ) return;
+		mouseActionString = "Clear pin layer preference";
+	}
+	else if ( !bSmartPan && GetDefiningRect() )
 	{
 		m_board.GetRectMgr().UpdateNewRect(m_gridRow, m_gridCol);
 		SelectAllInRects();
 	}
-	else if ( GetCtrlKeyDown() )
-	{
-		int pixmapX(0), pixmapY(0);
-		GetPixMapXY(event->pos(), pixmapX, pixmapY);
-
-		if ( deltaRow == 0 ) { if ( pixmapY > W * m_board.GetRows() ) deltaRow = 1; }
-		if ( deltaCol == 0 ) { if ( pixmapX > W * m_board.GetCols() ) deltaCol = 1; }
-		if ( deltaRow == 0 ) { if ( pixmapY < W ) deltaRow = -1; }
-		if ( deltaCol == 0 ) { if ( pixmapX < W ) deltaCol = -1; }
-		if ( deltaRow == 0 && deltaCol == 0 ) return;	// No change
-		m_board.SmartPan(deltaRow, deltaCol);	// Pan whole circuit w.r.t. grid area, growing/shrinking as needed
-		mouseActionString = "Smart move/grow/crop";
-	}
-	else if ( GetPaintBoard() )	// (Un)Paint nodeId on board but NOT pins
+	else if ( !bSmartPan && GetPaintBoard() && trackMode != TRACKMODE::OFF )	// (Un)Paint nodeId on board but NOT pins
 	{
 		assert( !GetPaintPins() && !GetPaintFlood() );	// Sanity check
-		if ( trackMode == TRACKMODE::OFF ) return;
 		if ( m_bLeftClick )		// Paint
 		{
 			const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, GetCurrentNodeId(), GetPaintPins());	// Only allow paint board (not pins)
@@ -391,7 +418,7 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 		m_board.WipeAutoSetPoints();
 		m_board.PlaceFloaters();	// See if we can now place floating components down
 	}
-	else if ( GetCurrentTextId() != BAD_TEXTID )
+	else if ( !bSmartPan && GetCurrentTextId() != BAD_TEXTID )
 	{
 		int pixmapX(0), pixmapY(0);
 		GetPixMapXY(event->pos(), pixmapX, pixmapY);
@@ -424,10 +451,8 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 		}
 		mouseActionString = ( GetResizingText() ) ? "Resize text box" : "Move text box";
 	}
-	else if ( GetCurrentCompId() != BAD_COMPID )	// Move user-group components
+	else if ( !bSmartPan && GetCurrentCompId() != BAD_COMPID && compMode != COMPSMODE::OFF )	// Move user-group components
 	{
-		if ( compMode == COMPSMODE::OFF ) return;
-
 		int pixmapX(0), pixmapY(0);
 		GetPixMapXY(event->pos(), pixmapX, pixmapY);
 
@@ -451,7 +476,20 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 		}
 		mouseActionString = "Move part(s)";
 	}
-	else 
+	else if ( bSmartPan || (ALLOW_SMART_PAN_WITHOUT_CTRLKEY && m_bRightClick) )
+	{
+		int pixmapX(0), pixmapY(0);
+		GetPixMapXY(event->pos(), pixmapX, pixmapY);
+
+		if ( deltaRow == 0 ) { if ( pixmapY > W * m_board.GetRows() ) deltaRow = 1; }
+		if ( deltaCol == 0 ) { if ( pixmapX > W * m_board.GetCols() ) deltaCol = 1; }
+		if ( deltaRow == 0 ) { if ( pixmapY < W ) deltaRow = -1; }
+		if ( deltaCol == 0 ) { if ( pixmapX < W ) deltaCol = -1; }
+		if ( deltaRow == 0 && deltaCol == 0 ) return;	// No change
+		m_board.SmartPan(deltaRow, deltaCol);	// Pan whole circuit w.r.t. grid area, growing/shrinking as needed
+		mouseActionString = "Smart move/grow/crop";
+	}
+	else
 		return;
 
 	if ( abs(deltaRow) <= 1 && abs(deltaCol) <= 1 )	// If not moved mouse too fast ...
@@ -494,7 +532,7 @@ void MainWindow::mouseReleaseEvent(QMouseEvent* event)
 		ShowCurrentRectSize();
 		return RepaintSkipRouting();
 	}
-	if ( GetPaintPins() || GetPaintBoard() || GetPaintFlood() )
+	if ( GetPaintPins() || GetPaintBoard() || GetPaintFlood() || GetPaintLyrPref() )
 		centralWidget()->setCursor(Qt::CrossCursor);
 	else
 		centralWidget()->setCursor(Qt::OpenHandCursor);
@@ -508,6 +546,34 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 {
 	commonKeyPressEvent(event);
 
+	// Pin offsets (Ctrl + Cursor keys)
+	if ( GetCtrlKeyDown() && !m_board.GetVeroTracks() && !m_board.GetCompEdit() )
+	{
+		const Element* pC =  m_board.Get(0, m_gridRow, m_gridCol);
+		if ( pC->GetHasPin() && !pC->GetHasWire() )	// Wires can share holes so cannot have offset pads
+		{
+			Component&		comp		= m_board.GetCompMgr().GetComponentById( pC->GetCompId() );
+			const size_t	pinIndex	= pC->GetPinIndex();
+			int dx(0), dy(0);
+			switch( event->key() )
+			{
+				case Qt::Key_Left:	 dx-=5; break;
+				case Qt::Key_Right:	 dx+=5; break;
+				case Qt::Key_Up:	 dy-=5; break;
+				case Qt::Key_Down:	 dy+=5; break;
+			}
+			comp.IncCompPinOffsets(pinIndex, dx, dy);
+
+			int X,Y;
+			comp.GetCompPinOffsets(pinIndex, X, Y);
+
+			char buffer[256] = {'\0'};
+			sprintf(buffer,"(X, Y) pad offset (mil) = (%d, %d)", X, Y);
+			ui->statusBar->showMessage(QString(buffer), 1000);
+
+			return RepaintSkipRouting();
+		}
+	}
 	if ( GetCtrlKeyDown() ) return;		// Try to keep Ctrl key input handled by menu items
 	if ( GetShiftKeyDown() ) return;	// Ignore other key presses while trying to group components
 
@@ -584,13 +650,15 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 		// Only one flag for paint-board/paint-pins/paint-flood must be true
 		switch( event->key() )
 		{
-			case Qt::Key_P:		if ( trackMode == TRACKMODE::OFF || compMode == COMPSMODE::OFF || GetPaintBoard() || GetPaintFlood() ) return;
+			case Qt::Key_P:		if ( trackMode == TRACKMODE::OFF || compMode == COMPSMODE::OFF || GetPaintBoard() || GetPaintFlood() || GetPaintLyrPref() ) return;
 								SetPaintPins(true);		centralWidget()->setCursor(Qt::CrossCursor); break;
-			case Qt::Key_Space:	if ( trackMode == TRACKMODE::OFF || GetPaintPins() || GetPaintFlood() ) return;
+			case Qt::Key_Space:	if ( trackMode == TRACKMODE::OFF || GetPaintPins() || GetPaintFlood() || GetPaintLyrPref() ) return;
 								SetPaintBoard(true);	centralWidget()->setCursor(Qt::CrossCursor); break;
-			case Qt::Key_F:		if ( trackMode == TRACKMODE::OFF || compMode == COMPSMODE::OFF || GetPaintBoard() || GetPaintPins() ) return;
+			case Qt::Key_F:		if ( trackMode == TRACKMODE::OFF || compMode == COMPSMODE::OFF || GetPaintBoard() || GetPaintPins() || GetPaintLyrPref() ) return;
 								if ( m_board.GetRoutingEnabled() ) return;
 								SetPaintFlood(true);	centralWidget()->setCursor(Qt::CrossCursor); break;
+			case Qt::Key_T:		if ( trackMode != TRACKMODE::PCB || m_board.GetLyrs() == 1 || GetPaintBoard() || GetPaintPins() || GetPaintFlood() ) return;
+								SetPaintLyrPref(true);	centralWidget()->setCursor(Qt::CrossCursor); break;
 			case Qt::Key_W:		WipeTracks();	break;
 		}
 	}
@@ -620,6 +688,7 @@ void MainWindow::keyReleaseEvent(QKeyEvent* event)
 			case Qt::Key_R:		SetDefiningRect(false);	break;
 			case Qt::Key_P:		SetPaintPins(false);	break;
 			case Qt::Key_F:		SetPaintFlood(false);	break;
+			case Qt::Key_T:		SetPaintLyrPref(false);	break;
 			case Qt::Key_Space:	SetPaintBoard(false);	break;
 		}
 		if ( GetCurrentTextId() != BAD_TEXTID && m_bMouseClick )
